@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,7 +21,9 @@ import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
 import java.lang.annotation.Target;
 import java.lang.reflect.Method;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.Iterator;
 
 import org.junit.Rule;
@@ -31,42 +33,32 @@ import org.junit.rules.ExpectedException;
 import org.springframework.cache.interceptor.CacheEvictOperation;
 import org.springframework.cache.interceptor.CacheOperation;
 import org.springframework.cache.interceptor.CacheableOperation;
-import org.springframework.util.ReflectionUtils;
+import org.springframework.core.annotation.AliasFor;
 
+import static org.hamcrest.CoreMatchers.*;
 import static org.junit.Assert.*;
 
 /**
  * @author Costin Leau
  * @author Stephane Nicoll
+ * @author Sam Brannen
  */
 public class AnnotationCacheOperationSourceTests {
 
 	@Rule
-	public final ExpectedException thrown = ExpectedException.none();
+	public final ExpectedException exception = ExpectedException.none();
 
-	private AnnotationCacheOperationSource source = new AnnotationCacheOperationSource();
+	private final AnnotationCacheOperationSource source = new AnnotationCacheOperationSource();
 
-	private Collection<CacheOperation> getOps(Class<?> target, String name,
-											  int expectedNumberOfOperations) {
-		Collection<CacheOperation> result = getOps(target, name);
-		assertEquals("Wrong number of operation(s) for '"+name+"'",
-				expectedNumberOfOperations, result.size());
-		return result;
-	}
-
-	private Collection<CacheOperation> getOps(Class<?> target, String name) {
-		Method method = ReflectionUtils.findMethod(target, name);
-		return source.getCacheOperations(method, target);
-	}
 
 	@Test
-	public void testSingularAnnotation() throws Exception {
+	public void singularAnnotation() throws Exception {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "singular", 1);
 		assertTrue(ops.iterator().next() instanceof CacheableOperation);
 	}
 
 	@Test
-	public void testMultipleAnnotation() throws Exception {
+	public void multipleAnnotation() throws Exception {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "multiple", 2);
 		Iterator<CacheOperation> it = ops.iterator();
 		assertTrue(it.next() instanceof CacheableOperation);
@@ -74,7 +66,7 @@ public class AnnotationCacheOperationSourceTests {
 	}
 
 	@Test
-	public void testCaching() throws Exception {
+	public void caching() throws Exception {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "caching", 2);
 		Iterator<CacheOperation> it = ops.iterator();
 		assertTrue(it.next() instanceof CacheableOperation);
@@ -82,13 +74,18 @@ public class AnnotationCacheOperationSourceTests {
 	}
 
 	@Test
-	public void testSingularStereotype() throws Exception {
+	public void emptyCaching() throws Exception {
+		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "emptyCaching", 0);
+	}
+
+	@Test
+	public void singularStereotype() throws Exception {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "singleStereotype", 1);
 		assertTrue(ops.iterator().next() instanceof CacheEvictOperation);
 	}
 
 	@Test
-	public void testMultipleStereotypes() throws Exception {
+	public void multipleStereotypes() throws Exception {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "multipleStereotype", 3);
 		Iterator<CacheOperation> it = ops.iterator();
 		assertTrue(it.next() instanceof CacheableOperation);
@@ -101,65 +98,99 @@ public class AnnotationCacheOperationSourceTests {
 	}
 
 	@Test
-	public void testCustomKeyGenerator() {
+	public void singleComposedAnnotation() throws Exception {
+		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "singleComposed", 2);
+		Iterator<CacheOperation> it = ops.iterator();
+
+		CacheOperation cacheOperation = it.next();
+		assertThat(cacheOperation, instanceOf(CacheableOperation.class));
+		assertThat(cacheOperation.getCacheNames(), equalTo(Collections.singleton("directly declared")));
+		assertThat(cacheOperation.getKey(), equalTo(""));
+
+		cacheOperation = it.next();
+		assertThat(cacheOperation, instanceOf(CacheableOperation.class));
+		assertThat(cacheOperation.getCacheNames(), equalTo(Collections.singleton("composedCache")));
+		assertThat(cacheOperation.getKey(), equalTo("composedKey"));
+	}
+
+	@Test
+	public void multipleComposedAnnotations() throws Exception {
+		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "multipleComposed", 4);
+		Iterator<CacheOperation> it = ops.iterator();
+
+		CacheOperation cacheOperation = it.next();
+		assertThat(cacheOperation, instanceOf(CacheableOperation.class));
+		assertThat(cacheOperation.getCacheNames(), equalTo(Collections.singleton("directly declared")));
+		assertThat(cacheOperation.getKey(), equalTo(""));
+
+		cacheOperation = it.next();
+		assertThat(cacheOperation, instanceOf(CacheableOperation.class));
+		assertThat(cacheOperation.getCacheNames(), equalTo(Collections.singleton("composedCache")));
+		assertThat(cacheOperation.getKey(), equalTo("composedKey"));
+
+		cacheOperation = it.next();
+		assertThat(cacheOperation, instanceOf(CacheableOperation.class));
+		assertThat(cacheOperation.getCacheNames(), equalTo(Collections.singleton("foo")));
+		assertThat(cacheOperation.getKey(), equalTo(""));
+
+		cacheOperation = it.next();
+		assertThat(cacheOperation, instanceOf(CacheEvictOperation.class));
+		assertThat(cacheOperation.getCacheNames(), equalTo(Collections.singleton("composedCacheEvict")));
+		assertThat(cacheOperation.getKey(), equalTo("composedEvictionKey"));
+	}
+
+	@Test
+	public void customKeyGenerator() {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "customKeyGenerator", 1);
 		CacheOperation cacheOperation = ops.iterator().next();
 		assertEquals("Custom key generator not set", "custom", cacheOperation.getKeyGenerator());
 	}
 
 	@Test
-	public void testCustomKeyGeneratorInherited() {
+	public void customKeyGeneratorInherited() {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "customKeyGeneratorInherited", 1);
 		CacheOperation cacheOperation = ops.iterator().next();
 		assertEquals("Custom key generator not set", "custom", cacheOperation.getKeyGenerator());
 	}
 
 	@Test
-	public void testKeyAndKeyGeneratorCannotBeSetTogether() {
-		try {
-			getOps(AnnotatedClass.class, "invalidKeyAndKeyGeneratorSet");
-			fail("Should have failed to parse @Cacheable annotation");
-		} catch (IllegalStateException e) {
-			// expected
-		}
+	public void keyAndKeyGeneratorCannotBeSetTogether() {
+		exception.expect(IllegalStateException.class);
+		getOps(AnnotatedClass.class, "invalidKeyAndKeyGeneratorSet");
 	}
 
 	@Test
-	public void testCustomCacheManager() {
+	public void customCacheManager() {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "customCacheManager", 1);
 		CacheOperation cacheOperation = ops.iterator().next();
 		assertEquals("Custom cache manager not set", "custom", cacheOperation.getCacheManager());
 	}
 
 	@Test
-	public void testCustomCacheManagerInherited() {
+	public void customCacheManagerInherited() {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "customCacheManagerInherited", 1);
 		CacheOperation cacheOperation = ops.iterator().next();
 		assertEquals("Custom cache manager not set", "custom", cacheOperation.getCacheManager());
 	}
 
 	@Test
-	public void testCustomCacheResolver() {
+	public void customCacheResolver() {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "customCacheResolver", 1);
 		CacheOperation cacheOperation = ops.iterator().next();
 		assertEquals("Custom cache resolver not set", "custom", cacheOperation.getCacheResolver());
 	}
 
 	@Test
-	public void testCustomCacheResolverInherited() {
+	public void customCacheResolverInherited() {
 		Collection<CacheOperation> ops = getOps(AnnotatedClass.class, "customCacheResolverInherited", 1);
 		CacheOperation cacheOperation = ops.iterator().next();
 		assertEquals("Custom cache resolver not set", "custom", cacheOperation.getCacheResolver());
 	}
 
 	@Test
-	public void testCacheResolverAndCacheManagerCannotBeSetTogether() {
-		try {
-			getOps(AnnotatedClass.class, "invalidCacheResolverAndCacheManagerSet");
-			fail("Should have failed to parse @Cacheable annotation");
-		} catch (IllegalStateException e) {
-			// expected
-		}
+	public void cacheResolverAndCacheManagerCannotBeSetTogether() {
+		exception.expect(IllegalStateException.class);
+		getOps(AnnotatedClass.class, "invalidCacheResolverAndCacheManagerSet");
 	}
 
 	@Test
@@ -234,19 +265,38 @@ public class AnnotationCacheOperationSourceTests {
 		assertSharedConfig(cacheOperation, "classKeyGenerator", "classCacheManager", "", "classCacheName");
 	}
 
+
+	private Collection<CacheOperation> getOps(Class<?> target, String name, int expectedNumberOfOperations) {
+		Collection<CacheOperation> result = getOps(target, name);
+		assertEquals("Wrong number of operation(s) for '" + name + "'", expectedNumberOfOperations, result.size());
+		return result;
+	}
+
+	private Collection<CacheOperation> getOps(Class<?> target, String name) {
+		try {
+			Method method = target.getMethod(name);
+			return source.getCacheOperations(method, target);
+		}
+		catch (NoSuchMethodException ex) {
+			throw new IllegalStateException(ex);
+		}
+	}
+
 	private void assertSharedConfig(CacheOperation actual, String keyGenerator, String cacheManager,
-									String cacheResolver, String... cacheNames) {
+			String cacheResolver, String... cacheNames) {
+
 		assertEquals("Wrong key manager",  keyGenerator, actual.getKeyGenerator());
 		assertEquals("Wrong cache manager", cacheManager, actual.getCacheManager());
 		assertEquals("Wrong cache resolver", cacheResolver, actual.getCacheResolver());
-		for (String cacheName : cacheNames) {
-			assertTrue("Cache '"+cacheName+"' not found (got "+actual.getCacheNames(),
-					actual.getCacheNames().contains(cacheName));
-		}
-		assertEquals("Wrong number of cache name(s)", cacheNames.length, actual.getCacheNames().size());
+		assertEquals("Wrong number of cache names", cacheNames.length, actual.getCacheNames().size());
+		Arrays.stream(cacheNames).forEach(cacheName ->
+				assertTrue("Cache '" + cacheName + "' not found in " + actual.getCacheNames(),
+						actual.getCacheNames().contains(cacheName)));
 	}
 
+
 	private static class AnnotatedClass {
+
 		@Cacheable("test")
 		public void singular() {
 		}
@@ -256,19 +306,23 @@ public class AnnotationCacheOperationSourceTests {
 		public void multiple() {
 		}
 
-		@Caching(cacheable = {@Cacheable("test")}, evict = {@CacheEvict("test")})
+		@Caching(cacheable = @Cacheable("test"), evict = @CacheEvict("test"))
 		public void caching() {
 		}
 
-		@Cacheable(value = "test", keyGenerator = "custom")
+		@Caching
+		public void emptyCaching() {
+		}
+
+		@Cacheable(cacheNames = "test", keyGenerator = "custom")
 		public void customKeyGenerator() {
 		}
 
-		@Cacheable(value = "test", cacheManager = "custom")
+		@Cacheable(cacheNames = "test", cacheManager = "custom")
 		public void customCacheManager() {
 		}
 
-		@Cacheable(value = "test", cacheResolver = "custom")
+		@Cacheable(cacheNames = "test", cacheResolver = "custom")
 		public void customCacheResolver() {
 		}
 
@@ -282,7 +336,19 @@ public class AnnotationCacheOperationSourceTests {
 		public void multipleStereotype() {
 		}
 
-		@Caching(cacheable = {@Cacheable(value = "test", key = "a"), @Cacheable(value = "test", key = "b")})
+		@Cacheable("directly declared")
+		@ComposedCacheable(cacheNames = "composedCache", key = "composedKey")
+		public void singleComposed() {
+		}
+
+		@Cacheable("directly declared")
+		@ComposedCacheable(cacheNames = "composedCache", key = "composedKey")
+		@CacheableFoo
+		@ComposedCacheEvict(cacheNames = "composedCacheEvict", key = "composedEvictionKey")
+		public void multipleComposed() {
+		}
+
+		@Caching(cacheable = { @Cacheable(cacheNames = "test", key = "a"), @Cacheable(cacheNames = "test", key = "b") })
 		public void multipleCaching() {
 		}
 
@@ -290,7 +356,7 @@ public class AnnotationCacheOperationSourceTests {
 		public void customKeyGeneratorInherited() {
 		}
 
-		@Cacheable(value = "test", key = "#root.methodName", keyGenerator = "custom")
+		@Cacheable(cacheNames = "test", key = "#root.methodName", keyGenerator = "custom")
 		public void invalidKeyAndKeyGeneratorSet() {
 		}
 
@@ -302,7 +368,7 @@ public class AnnotationCacheOperationSourceTests {
 		public void customCacheResolverInherited() {
 		}
 
-		@Cacheable(value = "test", cacheManager = "custom", cacheResolver = "custom")
+		@Cacheable(cacheNames = "test", cacheManager = "custom", cacheResolver = "custom")
 		public void invalidCacheResolverAndCacheManagerSet() {
 		}
 
@@ -311,6 +377,7 @@ public class AnnotationCacheOperationSourceTests {
 		}
 	}
 
+
 	@CacheConfig(cacheNames = "classCacheName",
 			keyGenerator = "classKeyGenerator",
 			cacheManager = "classCacheManager", cacheResolver = "classCacheResolver")
@@ -333,6 +400,7 @@ public class AnnotationCacheOperationSourceTests {
 		}
 	}
 
+
 	@CacheConfigFoo
 	private static class AnnotatedClassWithCustomDefault {
 
@@ -341,6 +409,7 @@ public class AnnotationCacheOperationSourceTests {
 		}
 	}
 
+
 	@CacheConfig(cacheNames = "classCacheName",
 			keyGenerator = "classKeyGenerator",
 			cacheManager = "classCacheManager")
@@ -357,9 +426,9 @@ public class AnnotationCacheOperationSourceTests {
 		@Cacheable
 		public void noCustomization() {
 		}
-
 	}
 
+
 	@CacheConfigFoo
 	@CacheConfig(cacheNames = "myCache") // multiple sources
 	private static class MultipleCacheConfig {
@@ -369,46 +438,87 @@ public class AnnotationCacheOperationSourceTests {
 		}
 	}
 
+
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.METHOD)
 	@Cacheable("foo")
 	public @interface CacheableFoo {
 	}
 
+
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.METHOD)
-	@Cacheable(value = "foo", keyGenerator = "custom")
+	@Cacheable(cacheNames = "foo", keyGenerator = "custom")
 	public @interface CacheableFooCustomKeyGenerator {
 	}
 
+
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.METHOD)
-	@Cacheable(value = "foo", cacheManager = "custom")
+	@Cacheable(cacheNames = "foo", cacheManager = "custom")
 	public @interface CacheableFooCustomCacheManager {
 	}
 
+
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.METHOD)
-	@Cacheable(value = "foo", cacheResolver = "custom")
+	@Cacheable(cacheNames = "foo", cacheResolver = "custom")
 	public @interface CacheableFooCustomCacheResolver {
 	}
 
-	@Retention(RetentionPolicy.RUNTIME)
-	@Target(ElementType.METHOD)
-	@CacheEvict(value = "foo")
-	public @interface EvictFoo {
-	}
 
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.METHOD)
-	@CacheEvict(value = "bar")
+	@CacheEvict("foo")
+	public @interface EvictFoo {
+	}
+
+
+	@Retention(RetentionPolicy.RUNTIME)
+	@Target(ElementType.METHOD)
+	@CacheEvict("bar")
 	public @interface EvictBar {
 	}
 
+
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.TYPE)
 	@CacheConfig(keyGenerator = "classKeyGenerator",
-			cacheManager = "classCacheManager", cacheResolver = "classCacheResolver")
+			cacheManager = "classCacheManager",
+			cacheResolver = "classCacheResolver")
 	public @interface CacheConfigFoo {
 	}
-}
\ No newline at end of file
+
+
+	@Retention(RetentionPolicy.RUNTIME)
+	@Target({ ElementType.METHOD, ElementType.TYPE })
+	@Cacheable(cacheNames = "shadowed cache name", key = "shadowed key")
+	@interface ComposedCacheable {
+
+		@AliasFor(annotation = Cacheable.class)
+		String[] value() default {};
+
+		@AliasFor(annotation = Cacheable.class)
+		String[] cacheNames() default {};
+
+		@AliasFor(annotation = Cacheable.class)
+		String key() default "";
+	}
+
+
+	@Retention(RetentionPolicy.RUNTIME)
+	@Target({ ElementType.METHOD, ElementType.TYPE })
+	@CacheEvict(cacheNames = "shadowed cache name", key = "shadowed key")
+	@interface ComposedCacheEvict {
+
+		@AliasFor(annotation = CacheEvict.class)
+		String[] value() default {};
+
+		@AliasFor(annotation = CacheEvict.class)
+		String[] cacheNames() default {};
+
+		@AliasFor(annotation = CacheEvict.class)
+		String key() default "";
+	}
+
+}
