@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,6 +29,7 @@ import org.springframework.aop.scope.ScopedObject;
 import org.springframework.aop.scope.ScopedProxyUtils;
 import org.springframework.aop.support.AopUtils;
 import org.springframework.beans.factory.BeanCreationException;
+import org.springframework.beans.factory.BeanDefinitionStoreException;
 import org.springframework.beans.factory.FactoryBean;
 import org.springframework.beans.factory.NoSuchBeanDefinitionException;
 import org.springframework.beans.factory.annotation.Autowired;
@@ -41,6 +42,7 @@ import org.springframework.beans.factory.support.DefaultListableBeanFactory;
 import org.springframework.beans.factory.support.RootBeanDefinition;
 import org.springframework.context.ApplicationContext;
 import org.springframework.context.annotation.componentscan.simple.SimpleComponent;
+import org.springframework.core.annotation.Order;
 import org.springframework.core.env.StandardEnvironment;
 import org.springframework.core.io.DescriptiveResource;
 import org.springframework.stereotype.Component;
@@ -272,6 +274,72 @@ public class ConfigurationClassPostProcessorTests {
 		beanFactory.getBean("bar", TestBean.class);
 	}
 
+	@Test
+	public void postProcessorFailsOnImplicitOverrideIfOverridingIsNotAllowed() {
+		RootBeanDefinition rbd = new RootBeanDefinition(TestBean.class);
+		rbd.setResource(new DescriptiveResource("XML or something"));
+		beanFactory.registerBeanDefinition("bar", rbd);
+		beanFactory.registerBeanDefinition("config", new RootBeanDefinition(SingletonBeanConfig.class));
+		beanFactory.setAllowBeanDefinitionOverriding(false);
+		ConfigurationClassPostProcessor pp = new ConfigurationClassPostProcessor();
+		try {
+			pp.postProcessBeanFactory(beanFactory);
+			fail("Should have thrown BeanDefinitionStoreException");
+		}
+		catch (BeanDefinitionStoreException ex) {
+			assertTrue(ex.getMessage().contains("bar"));
+			assertTrue(ex.getMessage().contains("SingletonBeanConfig"));
+			assertTrue(ex.getMessage().contains(TestBean.class.getName()));
+		}
+	}
+
+	@Test
+	public void configurationClassesProcessedInCorrectOrder() {
+		beanFactory.registerBeanDefinition("config1", new RootBeanDefinition(OverridingSingletonBeanConfig.class));
+		beanFactory.registerBeanDefinition("config2", new RootBeanDefinition(SingletonBeanConfig.class));
+		ConfigurationClassPostProcessor pp = new ConfigurationClassPostProcessor();
+		pp.postProcessBeanFactory(beanFactory);
+
+		Foo foo = beanFactory.getBean(Foo.class);
+		assertTrue(foo instanceof ExtendedFoo);
+		Bar bar = beanFactory.getBean(Bar.class);
+		assertSame(foo, bar.foo);
+	}
+
+	@Test
+	public void configurationClassesWithValidOverridingForProgrammaticCall() {
+		beanFactory.registerBeanDefinition("config1", new RootBeanDefinition(OverridingAgainSingletonBeanConfig.class));
+		beanFactory.registerBeanDefinition("config2", new RootBeanDefinition(OverridingSingletonBeanConfig.class));
+		beanFactory.registerBeanDefinition("config3", new RootBeanDefinition(SingletonBeanConfig.class));
+		ConfigurationClassPostProcessor pp = new ConfigurationClassPostProcessor();
+		pp.postProcessBeanFactory(beanFactory);
+
+		Foo foo = beanFactory.getBean(Foo.class);
+		assertTrue(foo instanceof ExtendedAgainFoo);
+		Bar bar = beanFactory.getBean(Bar.class);
+		assertSame(foo, bar.foo);
+	}
+
+	@Test
+	public void configurationClassesWithInvalidOverridingForProgrammaticCall() {
+		beanFactory.registerBeanDefinition("config1", new RootBeanDefinition(InvalidOverridingSingletonBeanConfig.class));
+		beanFactory.registerBeanDefinition("config2", new RootBeanDefinition(OverridingSingletonBeanConfig.class));
+		beanFactory.registerBeanDefinition("config3", new RootBeanDefinition(SingletonBeanConfig.class));
+		ConfigurationClassPostProcessor pp = new ConfigurationClassPostProcessor();
+		pp.postProcessBeanFactory(beanFactory);
+
+		try {
+			beanFactory.getBean(Bar.class);
+			fail("Should have thrown BeanCreationException");
+		}
+		catch (BeanCreationException ex) {
+			assertTrue(ex.getMessage().contains("OverridingSingletonBeanConfig.foo"));
+			assertTrue(ex.getMessage().contains(ExtendedFoo.class.getName()));
+			assertTrue(ex.getMessage().contains(Foo.class.getName()));
+			assertTrue(ex.getMessage().contains("InvalidOverridingSingletonBeanConfig"));
+		}
+	}
+
 	@Test
 	public void scopedProxyTargetMarkedAsNonAutowireCandidate() {
 		AutowiredAnnotationBeanPostProcessor bpp = new AutowiredAnnotationBeanPostProcessor();
@@ -463,6 +531,36 @@ public class ConfigurationClassPostProcessorTests {
 		beanFactory.registerBeanDefinition("serviceBeanProvider", new RootBeanDefinition(ServiceBeanProvider.class));
 		new ConfigurationClassPostProcessor().postProcessBeanFactory(beanFactory);
 		beanFactory.preInstantiateSingletons();
+
+		beanFactory.getBean(ServiceBean.class);
+	}
+
+	@Test
+	public void testConfigWithDefaultMethods() {
+		AutowiredAnnotationBeanPostProcessor bpp = new AutowiredAnnotationBeanPostProcessor();
+		bpp.setBeanFactory(beanFactory);
+		beanFactory.addBeanPostProcessor(bpp);
+		beanFactory.addBeanPostProcessor(new CommonAnnotationBeanPostProcessor());
+		beanFactory.registerBeanDefinition("configClass", new RootBeanDefinition(ConcreteConfigWithDefaultMethods.class));
+		beanFactory.registerBeanDefinition("serviceBeanProvider", new RootBeanDefinition(ServiceBeanProvider.class));
+		new ConfigurationClassPostProcessor().postProcessBeanFactory(beanFactory);
+		beanFactory.preInstantiateSingletons();
+
+		beanFactory.getBean(ServiceBean.class);
+	}
+
+	@Test
+	public void testConfigWithDefaultMethodsUsingAsm() {
+		AutowiredAnnotationBeanPostProcessor bpp = new AutowiredAnnotationBeanPostProcessor();
+		bpp.setBeanFactory(beanFactory);
+		beanFactory.addBeanPostProcessor(bpp);
+		beanFactory.addBeanPostProcessor(new CommonAnnotationBeanPostProcessor());
+		beanFactory.registerBeanDefinition("configClass", new RootBeanDefinition(ConcreteConfigWithDefaultMethods.class.getName()));
+		beanFactory.registerBeanDefinition("serviceBeanProvider", new RootBeanDefinition(ServiceBeanProvider.class.getName()));
+		new ConfigurationClassPostProcessor().postProcessBeanFactory(beanFactory);
+		beanFactory.preInstantiateSingletons();
+
+		beanFactory.getBean(ServiceBean.class);
 	}
 
 	@Test
@@ -494,21 +592,28 @@ public class ConfigurationClassPostProcessorTests {
 	}
 
 	@Test
-	public void testPrototypeArgumentsThroughBeanMethodCall() {
+	public void testPrototypeArgumentThroughBeanMethodCall() {
 		ApplicationContext ctx = new AnnotationConfigApplicationContext(BeanArgumentConfigWithPrototype.class);
 		ctx.getBean(FooFactory.class).createFoo(new BarArgument());
 	}
 
 	@Test
-	public void testSingletonArgumentsThroughBeanMethodCall() {
+	public void testSingletonArgumentThroughBeanMethodCall() {
 		ApplicationContext ctx = new AnnotationConfigApplicationContext(BeanArgumentConfigWithSingleton.class);
 		ctx.getBean(FooFactory.class).createFoo(new BarArgument());
 	}
 
+	@Test
+	public void testNullArgumentThroughBeanMethodCall() {
+		ApplicationContext ctx = new AnnotationConfigApplicationContext(BeanArgumentConfigWithNull.class);
+		ctx.getBean("aFoo");
+	}
+
 
 	// -------------------------------------------------------------------------
 
 	@Configuration
+	@Order(1)
 	static class SingletonBeanConfig {
 
 		public @Bean Foo foo() {
@@ -520,9 +625,44 @@ public class ConfigurationClassPostProcessorTests {
 		}
 	}
 
+	@Configuration
+	@Order(2)
+	static class OverridingSingletonBeanConfig {
+
+		public @Bean ExtendedFoo foo() {
+			return new ExtendedFoo();
+		}
+
+		public @Bean Bar bar() {
+			return new Bar(foo());
+		}
+	}
+
+	@Configuration
+	static class OverridingAgainSingletonBeanConfig {
+
+		public @Bean ExtendedAgainFoo foo() {
+			return new ExtendedAgainFoo();
+		}
+	}
+
+	@Configuration
+	static class InvalidOverridingSingletonBeanConfig {
+
+		public @Bean Foo foo() {
+			return new Foo();
+		}
+	}
+
 	static class Foo {
 	}
 
+	static class ExtendedFoo extends Foo {
+	}
+
+	static class ExtendedAgainFoo extends ExtendedFoo {
+	}
+
 	static class Bar {
 
 		final Foo foo;
@@ -673,7 +813,7 @@ public class ConfigurationClassPostProcessorTests {
 	}
 
 	@Retention(RetentionPolicy.RUNTIME)
-	@Scope(value = "prototype")
+	@Scope(scopeName = "prototype")
 	public @interface PrototypeScoped {
 
 		ScopedProxyMode proxyMode() default ScopedProxyMode.TARGET_CLASS;
@@ -683,7 +823,7 @@ public class ConfigurationClassPostProcessorTests {
 	public static class ScopedProxyRepositoryConfiguration {
 
 		@Bean
-		@Scope(value = "prototype", proxyMode = ScopedProxyMode.TARGET_CLASS)
+		@Scope(scopeName = "prototype", proxyMode = ScopedProxyMode.TARGET_CLASS)
 		public Repository<String> stringRepo() {
 			return new Repository<String>() {
 				@Override
@@ -815,7 +955,7 @@ public class ConfigurationClassPostProcessorTests {
 	@ComponentScan(basePackages = "org.springframework.context.annotation.componentscan.simple")
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.TYPE)
-	public static @interface ComposedConfiguration {
+	public @interface ComposedConfiguration {
 	}
 
 	@ComposedConfiguration
@@ -826,7 +966,7 @@ public class ConfigurationClassPostProcessorTests {
 	@ComponentScan
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.TYPE)
-	public static @interface ComposedConfigurationWithAttributeOverrides {
+	public @interface ComposedConfigurationWithAttributeOverrides {
 
 		String[] basePackages() default {};
 
@@ -845,7 +985,7 @@ public class ConfigurationClassPostProcessorTests {
 	@ComposedConfigurationWithAttributeOverrides
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.TYPE)
-	public static @interface ComposedComposedConfigurationWithAttributeOverrides {
+	public @interface ComposedComposedConfigurationWithAttributeOverrides {
 
 		String[] basePackages() default {};
 	}
@@ -857,14 +997,14 @@ public class ConfigurationClassPostProcessorTests {
 	@ComponentScan
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.TYPE)
-	public static @interface MetaComponentScan {
+	public @interface MetaComponentScan {
 	}
 
 	@MetaComponentScan
 	@Configuration
 	@Retention(RetentionPolicy.RUNTIME)
 	@Target(ElementType.TYPE)
-	public static @interface MetaComponentScanConfigurationWithAttributeOverrides {
+	public @interface MetaComponentScanConfigurationWithAttributeOverrides {
 
 		String[] basePackages() default {};
 	}
@@ -923,6 +1063,47 @@ public class ConfigurationClassPostProcessorTests {
 		}
 	}
 
+	public interface BaseInterface {
+
+		ServiceBean serviceBean();
+	}
+
+	public interface BaseDefaultMethods extends BaseInterface {
+
+		@Bean
+		default ServiceBeanProvider provider() {
+			return new ServiceBeanProvider();
+		}
+
+		@Bean
+		@Override
+		default ServiceBean serviceBean() {
+			return provider().getServiceBean();
+		}
+	}
+
+	public interface DefaultMethodsConfig extends BaseDefaultMethods {
+
+	}
+
+	@Configuration
+	public static class ConcreteConfigWithDefaultMethods implements DefaultMethodsConfig {
+
+		@Autowired
+		private ServiceBeanProvider provider;
+
+		@Bean
+		@Override
+		public ServiceBeanProvider provider() {
+			return provider;
+		}
+
+		@PostConstruct
+		public void validate() {
+			Assert.notNull(provider);
+		}
+	}
+
 	@Primary
 	public static class ServiceBeanProvider {
 
@@ -972,7 +1153,7 @@ public class ConfigurationClassPostProcessorTests {
 
 		@Bean
 		@Scope("prototype")
-		public DependingFoo foo(final BarArgument bar) {
+		public DependingFoo foo(BarArgument bar) {
 			return new DependingFoo(bar);
 		}
 
@@ -980,7 +1161,7 @@ public class ConfigurationClassPostProcessorTests {
 		public FooFactory fooFactory() {
 			return new FooFactory() {
 				@Override
-				public DependingFoo createFoo(final BarArgument bar) {
+				public DependingFoo createFoo(BarArgument bar) {
 					return foo(bar);
 				}
 			};
@@ -991,7 +1172,7 @@ public class ConfigurationClassPostProcessorTests {
 	static class BeanArgumentConfigWithSingleton {
 
 		@Bean @Lazy
-		public DependingFoo foo(final BarArgument bar) {
+		public DependingFoo foo(BarArgument bar) {
 			return new DependingFoo(bar);
 		}
 
@@ -999,19 +1180,39 @@ public class ConfigurationClassPostProcessorTests {
 		public FooFactory fooFactory() {
 			return new FooFactory() {
 				@Override
-				public DependingFoo createFoo(final BarArgument bar) {
+				public DependingFoo createFoo(BarArgument bar) {
 					return foo(bar);
 				}
 			};
 		}
 	}
 
+	@Configuration
+	static class BeanArgumentConfigWithNull {
+
+		@Bean
+		public DependingFoo aFoo() {
+			return foo(null);
+		}
+
+		@Bean @Lazy
+		public DependingFoo foo(BarArgument bar) {
+			return new DependingFoo(bar);
+		}
+
+		@Bean
+		public BarArgument bar() {
+			return new BarArgument();
+		}
+	}
+
 	static class BarArgument {
 	}
 
 	static class DependingFoo {
 
 		DependingFoo(BarArgument bar) {
+			Assert.notNull(bar);
 		}
 	}
 
