@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,19 +16,22 @@
 
 package org.springframework.format.datetime.standard;
 
-import java.text.DateFormat;
+import java.time.Duration;
 import java.time.Instant;
 import java.time.LocalDate;
 import java.time.LocalDateTime;
 import java.time.LocalTime;
+import java.time.MonthDay;
+import java.time.Period;
+import java.time.YearMonth;
 import java.time.ZoneId;
 import java.time.format.DateTimeFormatter;
 import java.time.format.FormatStyle;
 import java.util.ArrayList;
+import java.util.Date;
 import java.util.GregorianCalendar;
 import java.util.List;
 import java.util.Locale;
-import java.util.TimeZone;
 
 import org.junit.After;
 import org.junit.Before;
@@ -55,6 +58,7 @@ public class DateTimeFormattingTests {
 
 	private DataBinder binder;
 
+
 	@Before
 	public void setUp() {
 		DateTimeFormatterRegistrar registrar = new DateTimeFormatterRegistrar();
@@ -64,7 +68,6 @@ public class DateTimeFormattingTests {
 	private void setUp(DateTimeFormatterRegistrar registrar) {
 		conversionService = new FormattingConversionService();
 		DefaultConversionService.addDefaultConverters(conversionService);
-
 		registrar.registerFormatters(conversionService);
 
 		DateTimeBean bean = new DateTimeBean();
@@ -84,6 +87,7 @@ public class DateTimeFormattingTests {
 		DateTimeContextHolder.setDateTimeContext(null);
 	}
 
+
 	@Test
 	public void testBindLocalDate() {
 		MutablePropertyValues propertyValues = new MutablePropertyValues();
@@ -120,7 +124,7 @@ public class DateTimeFormattingTests {
 	@Test
 	public void testBindLocalDateArray() {
 		MutablePropertyValues propertyValues = new MutablePropertyValues();
-		propertyValues.add("localDate", new String[]{"10/31/09"});
+		propertyValues.add("localDate", new String[] {"10/31/09"});
 		binder.bind(propertyValues);
 		assertEquals(0, binder.getBindingResult().getErrorCount());
 	}
@@ -235,19 +239,23 @@ public class DateTimeFormattingTests {
 	@Test
 	public void testBindLocalDateTime() {
 		MutablePropertyValues propertyValues = new MutablePropertyValues();
-		propertyValues.add("localDateTime", "10/31/09 12:00 PM");
+		propertyValues.add("localDateTime", LocalDateTime.of(2009, 10, 31, 12, 0));
 		binder.bind(propertyValues);
 		assertEquals(0, binder.getBindingResult().getErrorCount());
-		assertEquals("10/31/09 12:00 PM", binder.getBindingResult().getFieldValue("localDateTime"));
+		String value = binder.getBindingResult().getFieldValue("localDateTime").toString();
+		assertTrue(value.startsWith("10/31/09"));
+		assertTrue(value.endsWith("12:00 PM"));
 	}
 
 	@Test
 	public void testBindLocalDateTimeAnnotated() {
 		MutablePropertyValues propertyValues = new MutablePropertyValues();
-		propertyValues.add("localDateTimeAnnotated", "Oct 31, 2009 12:00:00 PM");
+		propertyValues.add("localDateTimeAnnotated", LocalDateTime.of(2009, 10, 31, 12, 0));
 		binder.bind(propertyValues);
 		assertEquals(0, binder.getBindingResult().getErrorCount());
-		assertEquals("Oct 31, 2009 12:00:00 PM", binder.getBindingResult().getFieldValue("localDateTimeAnnotated"));
+		String value = binder.getBindingResult().getFieldValue("localDateTimeAnnotated").toString();
+		assertTrue(value.startsWith("Oct 31, 2009"));
+		assertTrue(value.endsWith("12:00:00 PM"));
 	}
 
 	@Test
@@ -256,7 +264,9 @@ public class DateTimeFormattingTests {
 		propertyValues.add("localDateTime", new GregorianCalendar(2009, 9, 31, 12, 0));
 		binder.bind(propertyValues);
 		assertEquals(0, binder.getBindingResult().getErrorCount());
-		assertEquals("10/31/09 12:00 PM", binder.getBindingResult().getFieldValue("localDateTime"));
+		String value = binder.getBindingResult().getFieldValue("localDateTime").toString();
+		assertTrue(value.startsWith("10/31/09"));
+		assertTrue(value.endsWith("12:00 PM"));
 	}
 
 	@Test
@@ -265,10 +275,12 @@ public class DateTimeFormattingTests {
 		registrar.setDateTimeStyle(FormatStyle.MEDIUM);
 		setUp(registrar);
 		MutablePropertyValues propertyValues = new MutablePropertyValues();
-		propertyValues.add("localDateTime", "Oct 31, 2009 12:00:00 PM");
+		propertyValues.add("localDateTime", LocalDateTime.of(2009, 10, 31, 12, 0));
 		binder.bind(propertyValues);
 		assertEquals(0, binder.getBindingResult().getErrorCount());
-		assertEquals("Oct 31, 2009 12:00:00 PM", binder.getBindingResult().getFieldValue("localDateTime"));
+		String value = binder.getBindingResult().getFieldValue("localDateTime").toString();
+		assertTrue(value.startsWith("Oct 31, 2009"));
+		assertTrue(value.endsWith("12:00:00 PM"));
 	}
 
 	@Test
@@ -280,6 +292,14 @@ public class DateTimeFormattingTests {
 		assertEquals("10/31/09 12:00 PM", binder.getBindingResult().getFieldValue("dateTimeAnnotatedPattern"));
 	}
 
+	@Test
+	public void testBindDateTimeOverflow() {
+		MutablePropertyValues propertyValues = new MutablePropertyValues();
+		propertyValues.add("dateTimeAnnotatedPattern", "02/29/09 12:00 PM");
+		binder.bind(propertyValues);
+		assertEquals(1, binder.getBindingResult().getErrorCount());
+	}
+
 	@Test
 	public void testBindISODate() {
 		MutablePropertyValues propertyValues = new MutablePropertyValues();
@@ -318,13 +338,47 @@ public class DateTimeFormattingTests {
 
 	@Test
 	public void testBindInstantFromJavaUtilDate() throws Exception {
-		DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, Locale.US);
-		df.setTimeZone(TimeZone.getTimeZone("GMT"));
 		MutablePropertyValues propertyValues = new MutablePropertyValues();
-		propertyValues.add("instant", df.parse("10/31/09 12:00 PM"));
+		propertyValues.add("instant", new Date(109, 9, 31, 12, 0));
 		binder.bind(propertyValues);
 		assertEquals(0, binder.getBindingResult().getErrorCount());
-		assertTrue(binder.getBindingResult().getFieldValue("instant").toString().startsWith("2009-10-31T12:00"));
+		assertTrue(binder.getBindingResult().getFieldValue("instant").toString().startsWith("2009-10-31"));
+	}
+
+	@Test
+	public void testBindPeriod() {
+		MutablePropertyValues propertyValues = new MutablePropertyValues();
+		propertyValues.add("period", "P6Y3M1D");
+		binder.bind(propertyValues);
+		assertEquals(0, binder.getBindingResult().getErrorCount());
+		assertTrue(binder.getBindingResult().getFieldValue("period").toString().equals("P6Y3M1D"));
+	}
+
+	@Test
+	public void testBindDuration() {
+		MutablePropertyValues propertyValues = new MutablePropertyValues();
+		propertyValues.add("duration", "PT8H6M12.345S");
+		binder.bind(propertyValues);
+		assertEquals(0, binder.getBindingResult().getErrorCount());
+		assertTrue(binder.getBindingResult().getFieldValue("duration").toString().equals("PT8H6M12.345S"));
+	}
+
+	@Test
+	public void testBindYearMonth() {
+		MutablePropertyValues propertyValues = new MutablePropertyValues();
+		propertyValues.add("yearMonth", "2007-12");
+		binder.bind(propertyValues);
+		assertEquals(0, binder.getBindingResult().getErrorCount());
+		assertTrue(binder.getBindingResult().getFieldValue("yearMonth").toString().equals("2007-12"));
+	}
+
+	@Test
+	public void testBindMonthDay() {
+		MutablePropertyValues propertyValues = new MutablePropertyValues();
+		propertyValues.add("monthDay", "--12-03");
+		binder.bind(propertyValues);
+		assertEquals(0, binder.getBindingResult().getErrorCount());
+		assertTrue(binder.getBindingResult().getFieldValue("monthDay").toString().equals("--12-03"));
 	}
 
 
@@ -359,6 +413,14 @@ public class DateTimeFormattingTests {
 
 		private Instant instant;
 
+		private Period period;
+
+		private Duration duration;
+
+		private YearMonth yearMonth;
+
+		private MonthDay monthDay;
+
 		private final List<DateTimeBean> children = new ArrayList<DateTimeBean>();
 
 		public LocalDate getLocalDate() {
@@ -449,6 +511,38 @@ public class DateTimeFormattingTests {
 			this.instant = instant;
 		}
 
+		public Period getPeriod() {
+			return period;
+		}
+
+		public void setPeriod(Period period) {
+			this.period = period;
+		}
+
+		public Duration getDuration() {
+			return duration;
+		}
+
+		public void setDuration(Duration duration) {
+			this.duration = duration;
+		}
+
+		public YearMonth getYearMonth() {
+			return yearMonth;
+		}
+
+		public void setYearMonth(YearMonth yearMonth) {
+			this.yearMonth = yearMonth;
+		}
+
+		public MonthDay getMonthDay() {
+			return monthDay;
+		}
+
+		public void setMonthDay(MonthDay monthDay) {
+			this.monthDay = monthDay;
+		}
+
 		public List<DateTimeBean> getChildren() {
 			return children;
 		}
