@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,7 +28,6 @@ import java.util.Properties;
 import org.joda.time.DateTime;
 import org.joda.time.LocalDate;
 import org.joda.time.format.DateTimeFormat;
-
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
@@ -53,7 +52,7 @@ import org.springframework.format.annotation.NumberFormat;
 import org.springframework.format.datetime.joda.DateTimeParser;
 import org.springframework.format.datetime.joda.JodaDateTimeFormatAnnotationFormatterFactory;
 import org.springframework.format.datetime.joda.ReadablePartialPrinter;
-import org.springframework.format.number.NumberFormatter;
+import org.springframework.format.number.NumberStyleFormatter;
 
 import static org.junit.Assert.*;
 
@@ -67,6 +66,7 @@ public class FormattingConversionServiceTests {
 
 	private FormattingConversionService formattingService;
 
+
 	@Before
 	public void setUp() {
 		formattingService = new FormattingConversionService();
@@ -79,9 +79,10 @@ public class FormattingConversionServiceTests {
 		LocaleContextHolder.setLocale(null);
 	}
 
+
 	@Test
 	public void testFormatFieldForTypeWithFormatter() throws ParseException {
-		formattingService.addFormatterForFieldType(Number.class, new NumberFormatter());
+		formattingService.addFormatterForFieldType(Number.class, new NumberStyleFormatter());
 		String formatted = formattingService.convert(3, String.class);
 		assertEquals("3", formatted);
 		Integer i = formattingService.convert("3", Integer.class);
@@ -217,7 +218,7 @@ public class FormattingConversionServiceTests {
 		assertEquals(new LocalDate(2009, 11, 1), new LocalDate(dates.get(1)));
 		assertEquals(new LocalDate(2009, 11, 2), new LocalDate(dates.get(2)));
 
-		Object model = BeanUtils.instantiate(modelClass);
+		Object model = modelClass.newInstance();
 		ConfigurablePropertyAccessor accessor = directFieldAccess ? PropertyAccessorFactory.forDirectFieldAccess(model) :
 				PropertyAccessorFactory.forBeanPropertyAccess(model);
 		accessor.setConversionService(formattingService);
@@ -239,26 +240,26 @@ public class FormattingConversionServiceTests {
 
 	@Test
 	public void testPrintNull() throws ParseException {
-		formattingService.addFormatterForFieldType(Number.class, new NumberFormatter());
+		formattingService.addFormatterForFieldType(Number.class, new NumberStyleFormatter());
 		assertEquals("", formattingService.convert(null, TypeDescriptor.valueOf(Integer.class), TypeDescriptor.valueOf(String.class)));
 	}
 
 	@Test
 	public void testParseNull() throws ParseException {
-		formattingService.addFormatterForFieldType(Number.class, new NumberFormatter());
+		formattingService.addFormatterForFieldType(Number.class, new NumberStyleFormatter());
 		assertNull(formattingService
 				.convert(null, TypeDescriptor.valueOf(String.class), TypeDescriptor.valueOf(Integer.class)));
 	}
 
 	@Test
 	public void testParseEmptyString() throws ParseException {
-		formattingService.addFormatterForFieldType(Number.class, new NumberFormatter());
+		formattingService.addFormatterForFieldType(Number.class, new NumberStyleFormatter());
 		assertNull(formattingService.convert("", TypeDescriptor.valueOf(String.class), TypeDescriptor.valueOf(Integer.class)));
 	}
 
 	@Test
 	public void testParseBlankString() throws ParseException {
-		formattingService.addFormatterForFieldType(Number.class, new NumberFormatter());
+		formattingService.addFormatterForFieldType(Number.class, new NumberStyleFormatter());
 		assertNull(formattingService.convert("     ", TypeDescriptor.valueOf(String.class), TypeDescriptor.valueOf(Integer.class)));
 	}
 
@@ -270,7 +271,7 @@ public class FormattingConversionServiceTests {
 
 	@Test(expected=ConversionFailedException.class)
 	public void testParseNullPrimitiveProperty() throws ParseException {
-		formattingService.addFormatterForFieldType(Integer.class, new NumberFormatter());
+		formattingService.addFormatterForFieldType(Integer.class, new NumberStyleFormatter());
 		assertNull(formattingService.convert(null, TypeDescriptor.valueOf(String.class), TypeDescriptor.valueOf(int.class)));
 	}
 
