@@ -17,6 +17,8 @@
 package org.springframework.jmx.export;
 
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
@@ -32,13 +34,15 @@ import javax.management.ObjectInstance;
 import javax.management.ObjectName;
 import javax.management.modelmbean.ModelMBeanInfo;
 
+import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.ExpectedException;
 
 import org.springframework.aop.framework.ProxyFactory;
 import org.springframework.beans.factory.support.BeanDefinitionBuilder;
 import org.springframework.beans.factory.support.DefaultListableBeanFactory;
-import org.springframework.beans.factory.xml.XmlBeanDefinitionReader;
-import org.springframework.core.io.ClassPathResource;
+import org.springframework.context.ConfigurableApplicationContext;
+import org.springframework.context.support.ClassPathXmlApplicationContext;
 import org.springframework.jmx.AbstractMBeanServerTests;
 import org.springframework.jmx.IJmxTestBean;
 import org.springframework.jmx.JmxTestBean;
@@ -62,10 +66,14 @@ import static org.junit.Assert.*;
  * @author Mark Fisher
  * @author Chris Beams
  * @author Sam Brannen
+ * @author Stephane Nicoll
  */
 @SuppressWarnings("deprecation")
 public final class MBeanExporterTests extends AbstractMBeanServerTests {
 
+	@Rule
+	public final ExpectedException thrown = ExpectedException.none();
+
 	private static final String OBJECT_NAME = "spring:test=jmxMBeanAdaptor";
 
 
@@ -75,12 +83,9 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		// put null in as a value...
 		listeners.put("*", null);
 		MBeanExporter exporter = new MBeanExporter();
-		try {
-			exporter.setNotificationListenerMappings(listeners);
-			fail("Must have thrown an IllegalArgumentException when registering a null instance as a NotificationListener.");
-		}
-		catch (IllegalArgumentException expected) {
-		}
+
+		thrown.expect(IllegalArgumentException.class);
+		exporter.setNotificationListenerMappings(listeners);
 	}
 
 	@Test
@@ -99,8 +104,9 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setServer(server);
 		exporter.setNotificationListenerMappings(listeners);
 		try {
-			exporter.afterPropertiesSet();
-			fail("Must have thrown an MBeanExportException when registering a NotificationListener on a non-existent MBean.");
+			start(exporter);
+			fail("Must have thrown an MBeanExportException when registering a " +
+					"NotificationListener on a non-existent MBean.");
 		}
 		catch (MBeanExportException expected) {
 			assertTrue(expected.contains(InstanceNotFoundException.class));
@@ -112,22 +118,16 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		MBeanExporter exporter = new MBeanExporter();
 		exporter.setBeans(getBeanMap());
 		exporter.setServer(server);
-		exporter.afterPropertiesSet();
-		assertIsRegistered("The bean was not registered with the MBeanServer",
-				ObjectNameManager.getInstance(OBJECT_NAME));
+		try {
+			start(exporter);
+			assertIsRegistered("The bean was not registered with the MBeanServer",
+					ObjectNameManager.getInstance(OBJECT_NAME));
+		}
+		finally {
+   			exporter.destroy();
+		}
 	}
 
-	/** Fails if JVM platform MBean server has been started already
-	@Test
-	public void testWithLocatedMBeanServer() throws Exception {
-		MBeanExporter adaptor = new MBeanExporter();
-		adaptor.setBeans(getBeanMap());
-		adaptor.afterPropertiesSet();
-		assertIsRegistered("The bean was not registered with the MBeanServer", ObjectNameManager.getInstance(OBJECT_NAME));
-		server.unregisterMBean(new ObjectName(OBJECT_NAME));
-	}
-	*/
-
 	@Test
 	public void testUserCreatedMBeanRegWithDynamicMBean() throws Exception {
 		Map<String, Object> map = new HashMap<String, Object>();
@@ -139,20 +139,24 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setServer(server);
 		exporter.setBeans(map);
 		exporter.setAssembler(asm);
-		exporter.afterPropertiesSet();
 
-		Object name = server.getAttribute(ObjectNameManager.getInstance("spring:name=dynBean"), "Name");
-		assertEquals("The name attribute is incorrect", "Rob Harrop", name);
-		assertFalse("Assembler should not have been invoked", asm.invoked);
+		try {
+			start(exporter);
+			Object name = server.getAttribute(ObjectNameManager.getInstance("spring:name=dynBean"), "Name");
+			assertEquals("The name attribute is incorrect", "Rob Harrop", name);
+			assertFalse("Assembler should not have been invoked", asm.invoked);
+		}
+		finally {
+   			exporter.destroy();
+		}
 	}
 
 	@Test
 	public void testAutodetectMBeans() throws Exception {
-		DefaultListableBeanFactory bf = new DefaultListableBeanFactory();
-		new XmlBeanDefinitionReader(bf).loadBeanDefinitions(new ClassPathResource("autodetectMBeans.xml", getClass()));
+		ConfigurableApplicationContext ctx = load("autodetectMBeans.xml");
 		try {
-			bf.getBean("exporter");
-			MBeanServer server = (MBeanServer) bf.getBean("server");
+			ctx.getBean("exporter");
+			MBeanServer server = ctx.getBean("server", MBeanServer.class);
 			ObjectInstance instance = server.getObjectInstance(ObjectNameManager.getInstance("spring:mbean=true"));
 			assertNotNull(instance);
 			instance = server.getObjectInstance(ObjectNameManager.getInstance("spring:mbean2=true"));
@@ -161,39 +165,33 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 			assertNotNull(instance);
 		}
 		finally {
-			bf.destroySingletons();
+			ctx.close();
 		}
 	}
 
 	@Test
 	public void testAutodetectWithExclude() throws Exception {
-		DefaultListableBeanFactory bf = new DefaultListableBeanFactory();
-		new XmlBeanDefinitionReader(bf).loadBeanDefinitions(new ClassPathResource("autodetectMBeans.xml", getClass()));
+		ConfigurableApplicationContext ctx = load("autodetectMBeans.xml");
 		try {
-			bf.getBean("exporter");
-			MBeanServer server = (MBeanServer) bf.getBean("server");
+			ctx.getBean("exporter");
+			MBeanServer server = ctx.getBean("server", MBeanServer.class);
 			ObjectInstance instance = server.getObjectInstance(ObjectNameManager.getInstance("spring:mbean=true"));
 			assertNotNull(instance);
 
-			try {
-				server.getObjectInstance(ObjectNameManager.getInstance("spring:mbean=false"));
-				fail("MBean with name spring:mbean=false should have been excluded");
-			}
-			catch (InstanceNotFoundException expected) {
-			}
+			thrown.expect(InstanceNotFoundException.class);
+			server.getObjectInstance(ObjectNameManager.getInstance("spring:mbean=false"));
 		}
 		finally {
-			bf.destroySingletons();
+			ctx.close();
 		}
 	}
 
 	@Test
 	public void testAutodetectLazyMBeans() throws Exception {
-		DefaultListableBeanFactory bf = new DefaultListableBeanFactory();
-		new XmlBeanDefinitionReader(bf).loadBeanDefinitions(new ClassPathResource("autodetectLazyMBeans.xml", getClass()));
+		ConfigurableApplicationContext ctx = load("autodetectLazyMBeans.xml");
 		try {
-			bf.getBean("exporter");
-			MBeanServer server = (MBeanServer) bf.getBean("server");
+			ctx.getBean("exporter");
+			MBeanServer server = ctx.getBean("server", MBeanServer.class);
 
 			ObjectName oname = ObjectNameManager.getInstance("spring:mbean=true");
 			assertNotNull(server.getObjectInstance(oname));
@@ -206,19 +204,18 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 			assertEquals("Invalid name returned", "Juergen Hoeller", name);
 		}
 		finally {
-			bf.destroySingletons();
+			ctx.close();
 		}
 	}
 
 	@Test
 	public void testAutodetectNoMBeans() throws Exception {
-		DefaultListableBeanFactory bf = new DefaultListableBeanFactory();
-		new XmlBeanDefinitionReader(bf).loadBeanDefinitions(new ClassPathResource("autodetectNoMBeans.xml", getClass()));
+		ConfigurableApplicationContext ctx = load("autodetectNoMBeans.xml");
 		try {
-			bf.getBean("exporter");
+			ctx.getBean("exporter");
 		}
 		finally {
-			bf.destroySingletons();
+			ctx.close();
 		}
 	}
 
@@ -231,13 +228,14 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setBeans(getBeanMap());
 		exporter.setServer(server);
 		exporter.setListeners(listener1, listener2);
-		exporter.afterPropertiesSet();
+		start(exporter);
 		exporter.destroy();
 
 		assertListener(listener1);
 		assertListener(listener2);
 	}
 
+
 	@Test
 	public void testExportJdkProxy() throws Exception {
 		JmxTestBean bean = new JmxTestBean();
@@ -277,7 +275,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setServer(server);
 		exporter.setBeans(beans);
 
-		exporter.afterPropertiesSet();
+		start(exporter);
 
 		ObjectInstance instance = server.getObjectInstance(objectName);
 		assertNotNull(instance);
@@ -306,7 +304,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setBeans(beans);
 		exporter.setRegistrationBehavior(MBeanExporter.REGISTRATION_IGNORE_EXISTING);
 
-		exporter.afterPropertiesSet();
+		start(exporter);
 
 		ObjectInstance instance = server.getObjectInstance(objectName);
 		assertNotNull(instance);
@@ -337,7 +335,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setBeans(beans);
 		exporter.setRegistrationPolicy(RegistrationPolicy.REPLACE_EXISTING);
 
-		exporter.afterPropertiesSet();
+		start(exporter);
 
 		ObjectInstance instance = server.getObjectInstance(objectName);
 		assertNotNull(instance);
@@ -362,7 +360,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setServer(getServer());
 		exporter.setBeans(beans);
 		exporter.setExposeManagedResourceClassLoader(true);
-		exporter.afterPropertiesSet();
+		start(exporter);
 
 		assertIsRegistered("Bean instance not registered", objectName);
 
@@ -392,7 +390,8 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setBeanFactory(factory);
 		exporter.setAutodetectMode(MBeanExporter.AUTODETECT_NONE);
 		// MBean has a bad ObjectName, so if said MBean is autodetected, an exception will be thrown...
-		exporter.afterPropertiesSet();
+		start(exporter);
+
 	}
 
 	@Test
@@ -408,7 +407,8 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setAssembler(new NamedBeanAutodetectCapableMBeanInfoAssemblerStub(exportedBeanName));
 		exporter.setBeanFactory(factory);
 		exporter.setAutodetectMode(MBeanExporter.AUTODETECT_MBEAN);
-		exporter.afterPropertiesSet();
+		start(exporter);
+
 		assertIsRegistered("Bona fide MBean not autodetected in AUTODETECT_MBEAN mode",
 				ObjectNameManager.getInstance(OBJECT_NAME));
 		assertIsNotRegistered("Bean autodetected and (only) AUTODETECT_MBEAN mode is on",
@@ -430,7 +430,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setAssembler(new NamedBeanAutodetectCapableMBeanInfoAssemblerStub(exportedBeanName));
 		exporter.setBeanFactory(factory);
 		exporter.setAutodetectMode(MBeanExporter.AUTODETECT_ALL);
-		exporter.afterPropertiesSet();
+		start(exporter);
 		assertIsRegistered("Bona fide MBean not autodetected in (AUTODETECT_ALL) mode",
 				ObjectNameManager.getInstance(OBJECT_NAME));
 		assertIsRegistered("Bean not autodetected in (AUTODETECT_ALL) mode",
@@ -452,7 +452,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setAssembler(new NamedBeanAutodetectCapableMBeanInfoAssemblerStub(exportedBeanName));
 		exporter.setBeanFactory(factory);
 		exporter.setAutodetectMode(MBeanExporter.AUTODETECT_ASSEMBLER);
-		exporter.afterPropertiesSet();
+		start(exporter);
 		assertIsNotRegistered("Bona fide MBean was autodetected in AUTODETECT_ASSEMBLER mode - must not have been",
 				ObjectNameManager.getInstance(OBJECT_NAME));
 		assertIsRegistered("Bean not autodetected in AUTODETECT_ASSEMBLER mode",
@@ -476,101 +476,69 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setAssembler(new NamedBeanAutodetectCapableMBeanInfoAssemblerStub(OBJECT_NAME));
 		exporter.setBeanFactory(factory);
 		exporter.setAutodetectMode(MBeanExporter.AUTODETECT_ASSEMBLER);
-		exporter.afterPropertiesSet();
+		start(exporter);
 		assertIsRegistered("Explicitly exported bona fide MBean obviously not exported.",
 				ObjectNameManager.getInstance(OBJECT_NAME));
 	}
 
 	@Test
-	public void testSetAutodetectModeToOutOfRangeNegativeValue() throws Exception {
-		try {
-			MBeanExporter exporter = new MBeanExporter();
-			exporter.setAutodetectMode(-1);
-			fail("Must have failed when supplying an invalid negative out-of-range autodetect mode");
-		}
-		catch (IllegalArgumentException expected) {
-		}
+	public void testSetAutodetectModeToOutOfRangeNegativeValue() {
+		MBeanExporter exporter = new MBeanExporter();
+		thrown.expect(IllegalArgumentException.class);
+		exporter.setAutodetectMode(-1);
 	}
 
 	@Test
-	public void testSetAutodetectModeToOutOfRangePositiveValue() throws Exception {
-		try {
-			MBeanExporter exporter = new MBeanExporter();
-			exporter.setAutodetectMode(5);
-			fail("Must have failed when supplying an invalid positive out-of-range autodetect mode");
-		}
-		catch (IllegalArgumentException expected) {
-		}
+	public void testSetAutodetectModeToOutOfRangePositiveValue() {
+		MBeanExporter exporter = new MBeanExporter();
+		thrown.expect(IllegalArgumentException.class);
+		exporter.setAutodetectMode(5);
 	}
 
 	@Test
-	public void testSetAutodetectModeNameToNull() throws Exception {
-		try {
-			MBeanExporter exporter = new MBeanExporter();
-			exporter.setAutodetectModeName(null);
-			fail("Must have failed when supplying a null autodetect mode name");
-		}
-		catch (IllegalArgumentException expected) {
-		}
+	public void testSetAutodetectModeNameToNull() {
+		MBeanExporter exporter = new MBeanExporter();
+		thrown.expect(IllegalArgumentException.class);
+		exporter.setAutodetectModeName(null);
 	}
 
 	@Test
-	public void testSetAutodetectModeNameToAnEmptyString() throws Exception {
-		try {
-			MBeanExporter exporter = new MBeanExporter();
-			exporter.setAutodetectModeName("");
-			fail("Must have failed when supplying an empty autodetect mode name");
-		}
-		catch (IllegalArgumentException expected) {
-		}
+	public void testSetAutodetectModeNameToAnEmptyString() {
+		MBeanExporter exporter = new MBeanExporter();
+		thrown.expect(IllegalArgumentException.class);
+		exporter.setAutodetectModeName("");
 	}
 
 	@Test
-	public void testSetAutodetectModeNameToAWhitespacedString() throws Exception {
-		try {
-			MBeanExporter exporter = new MBeanExporter();
-			exporter.setAutodetectModeName("  \t");
-			fail("Must have failed when supplying a whitespace-only autodetect mode name");
-		}
-		catch (IllegalArgumentException expected) {
-		}
+	public void testSetAutodetectModeNameToAWhitespacedString() {
+		MBeanExporter exporter = new MBeanExporter();
+		thrown.expect(IllegalArgumentException.class);
+		exporter.setAutodetectModeName("  \t");
 	}
 
 	@Test
-	public void testSetAutodetectModeNameToARubbishValue() throws Exception {
-		try {
-			MBeanExporter exporter = new MBeanExporter();
-			exporter.setAutodetectModeName("That Hansel is... *sssooo* hot right now!");
-			fail("Must have failed when supplying a whitespace-only autodetect mode name");
-		}
-		catch (IllegalArgumentException expected) {
-		}
+	public void testSetAutodetectModeNameToARubbishValue() {
+		MBeanExporter exporter = new MBeanExporter();
+		thrown.expect(IllegalArgumentException.class);
+		exporter.setAutodetectModeName("That Hansel is... *sssooo* hot right now!");
 	}
 
 	@Test
 	public void testNotRunningInBeanFactoryAndPassedBeanNameToExport() throws Exception {
-		try {
-			MBeanExporter exporter = new MBeanExporter();
-			Map<String, Object> beans = new HashMap<String, Object>();
-			beans.put(OBJECT_NAME, "beanName");
-			exporter.setBeans(beans);
-			exporter.afterPropertiesSet();
-			fail("Expecting exception because MBeanExporter is not running in a BeanFactory and was passed bean name to (lookup and then) export");
-		}
-		catch (MBeanExportException expected) {
-		}
+		MBeanExporter exporter = new MBeanExporter();
+		Map<String, Object> beans = new HashMap<String, Object>();
+		beans.put(OBJECT_NAME, "beanName");
+		exporter.setBeans(beans);
+		thrown.expect(MBeanExportException.class);
+		start(exporter);
 	}
 
 	@Test
 	public void testNotRunningInBeanFactoryAndAutodetectionIsOn() throws Exception {
-		try {
-			MBeanExporter exporter = new MBeanExporter();
-			exporter.setAutodetectMode(MBeanExporter.AUTODETECT_ALL);
-			exporter.afterPropertiesSet();
-			fail("Expecting exception because MBeanExporter is not running in a BeanFactory and was configured to autodetect beans");
-		}
-		catch (MBeanExportException expected) {
-		}
+		MBeanExporter exporter = new MBeanExporter();
+		exporter.setAutodetectMode(MBeanExporter.AUTODETECT_ALL);
+		thrown.expect(MBeanExportException.class);
+		start(exporter);
 	}
 
 	/**
@@ -583,7 +551,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setServer(this.server);
 		MockMBeanExporterListener listener = new MockMBeanExporterListener();
 		exporter.setListeners(listener);
-		exporter.afterPropertiesSet();
+		start(exporter);
 		assertIsRegistered("The bean was not registered with the MBeanServer",
 				ObjectNameManager.getInstance(OBJECT_NAME));
 
@@ -615,7 +583,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		StubNotificationListener listener = new StubNotificationListener();
 		exporter.setNotificationListenerMappings(Collections.singletonMap(beanName, listener));
 
-		exporter.afterPropertiesSet();
+		start(exporter);
 	}
 
 	@Test
@@ -637,7 +605,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		StubNotificationListener listener = new StubNotificationListener();
 		exporter.setNotificationListenerMappings(Collections.singletonMap("*", listener));
 
-		exporter.afterPropertiesSet();
+		start(exporter);
 	}
 
 	/*
@@ -665,7 +633,7 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 		exporter.setBeanFactory(factory);
 
 		try {
-			exporter.afterPropertiesSet();
+			start(exporter);
 			fail("Must have failed during creation of RuntimeExceptionThrowingConstructorBean");
 		}
 		catch (RuntimeException expected) {
@@ -677,6 +645,32 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 				ObjectNameManager.getInstance(objectName2));
 	}
 
+	@Test
+	public void testIgnoreBeanName() throws MalformedObjectNameException {
+		DefaultListableBeanFactory factory = new DefaultListableBeanFactory();
+		String firstBeanName = "spring:type=TestBean";
+		factory.registerSingleton(firstBeanName, new TestBean("test"));
+		String secondBeanName = "spring:type=TestBean2";
+		factory.registerSingleton(secondBeanName, new TestBean("test2"));
+
+		MBeanExporter exporter = new MBeanExporter();
+		exporter.setServer(getServer());
+		exporter.setAssembler(new NamedBeanAutodetectCapableMBeanInfoAssemblerStub(firstBeanName, secondBeanName));
+		exporter.setBeanFactory(factory);
+		exporter.setAutodetectMode(MBeanExporter.AUTODETECT_ALL);
+		exporter.addExcludedBean(secondBeanName);
+
+		start(exporter);
+		assertIsRegistered("Bean not autodetected in (AUTODETECT_ALL) mode",
+				ObjectNameManager.getInstance(firstBeanName));
+		assertIsNotRegistered("Bean should have been excluded",
+				ObjectNameManager.getInstance(secondBeanName));
+	}
+
+	private ConfigurableApplicationContext load(String context) {
+		return new ClassPathXmlApplicationContext(context, getClass());
+	}
+
 	private Map<String, Object> getBeanMap() {
 		Map<String, Object> map = new HashMap<String, Object>();
 		map.put(OBJECT_NAME, new JmxTestBean());
@@ -793,15 +787,15 @@ public final class MBeanExporterTests extends AbstractMBeanServerTests {
 	private static final class NamedBeanAutodetectCapableMBeanInfoAssemblerStub extends
 			SimpleReflectiveMBeanInfoAssembler implements AutodetectCapableMBeanInfoAssembler {
 
-		private String namedBean;
+		private Collection<String> namedBeans;
 
-		public NamedBeanAutodetectCapableMBeanInfoAssemblerStub(String namedBean) {
-			this.namedBean = namedBean;
+		public NamedBeanAutodetectCapableMBeanInfoAssemblerStub(String... namedBeans) {
+			this.namedBeans = Arrays.asList(namedBeans);
 		}
 
 		@Override
 		public boolean includeBean(Class<?> beanClass, String beanName) {
-			return this.namedBean.equals(beanName);
+			return this.namedBeans.contains(beanName);
 		}
 	}
 
