@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -37,8 +37,10 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeSet;
+import java.util.concurrent.Callable;
 
 import org.hamcrest.Matchers;
 import org.junit.Rule;
@@ -60,6 +62,7 @@ import static org.mockito.BDDMockito.*;
  * Tests for {@link ResolvableType}.
  *
  * @author Phillip Webb
+ * @author Juergen Hoeller
  */
 @SuppressWarnings("rawtypes")
 @RunWith(MockitoJUnitRunner.class)
@@ -99,13 +102,64 @@ public class ResolvableTypeTests {
 	public void forClass() throws Exception {
 		ResolvableType type = ResolvableType.forClass(ExtendsList.class);
 		assertThat(type.getType(), equalTo((Type) ExtendsList.class));
+		assertThat(type.getRawClass(), equalTo(ExtendsList.class));
+		assertTrue(type.isAssignableFrom(ExtendsList.class));
+		assertFalse(type.isAssignableFrom(ArrayList.class));
 	}
 
 	@Test
-	public void forClassMustNotBeNull() throws Exception {
+	public void forClassWithNull() throws Exception {
+		ResolvableType type = ResolvableType.forClass(null);
+		assertThat(type.getType(), equalTo((Type) Object.class));
+		assertThat(type.getRawClass(), equalTo(Object.class));
+		assertTrue(type.isAssignableFrom(Object.class));
+		assertTrue(type.isAssignableFrom(String.class));
+	}
+
+	@Test
+	public void forRawClass() throws Exception {
+		ResolvableType type = ResolvableType.forRawClass(ExtendsList.class);
+		assertThat(type.getType(), equalTo((Type) ExtendsList.class));
+		assertThat(type.getRawClass(), equalTo(ExtendsList.class));
+		assertTrue(type.isAssignableFrom(ExtendsList.class));
+		assertFalse(type.isAssignableFrom(ArrayList.class));
+	}
+
+	@Test
+	public void forRawClassWithNull() throws Exception {
+		ResolvableType type = ResolvableType.forRawClass(null);
+		assertThat(type.getType(), equalTo((Type) Object.class));
+		assertThat(type.getRawClass(), equalTo(Object.class));
+		assertTrue(type.isAssignableFrom(Object.class));
+		assertTrue(type.isAssignableFrom(String.class));
+	}
+
+	@Test
+	public void forInstanceMustNotBeNull() {
 		this.thrown.expect(IllegalArgumentException.class);
-		this.thrown.expectMessage("Source class must not be null");
-		ResolvableType.forClass(null);
+		this.thrown.expectMessage("Instance must not be null");
+		ResolvableType.forInstance(null);
+	}
+
+	@Test
+	public void forInstanceNoProvider() {
+		ResolvableType type = ResolvableType.forInstance(new Object());
+		assertThat(type.getType(), equalTo(Object.class));
+		assertThat(type.resolve(), equalTo(Object.class));
+	}
+
+	@Test
+	public void forInstanceProvider() {
+		ResolvableType type = ResolvableType.forInstance(new MyGenericInterfaceType<String>(String.class));
+		assertThat(type.getRawClass(), equalTo(MyGenericInterfaceType.class));
+		assertThat(type.getGeneric().resolve(), equalTo(String.class));
+	}
+
+	@Test
+	public void forInstanceProviderNull() {
+		ResolvableType type = ResolvableType.forInstance(new MyGenericInterfaceType<String>(null));
+		assertThat(type.getType(), equalTo(MyGenericInterfaceType.class));
+		assertThat(type.resolve(), equalTo(MyGenericInterfaceType.class));
 	}
 
 	@Test
@@ -881,7 +935,7 @@ public class ResolvableTypeTests {
 	public void isAssignableFromMustNotBeNull() throws Exception {
 		this.thrown.expect(IllegalArgumentException.class);
 		this.thrown.expectMessage("Type must not be null");
-		ResolvableType.forClass(Object.class).isAssignableFrom(null);
+		ResolvableType.forClass(Object.class).isAssignableFrom((ResolvableType) null);
 	}
 
 	@Test
@@ -900,6 +954,20 @@ public class ResolvableTypeTests {
 		assertAssignable(objectType, objectType, charSequenceType, stringType).equalTo(true, true, true);
 		assertAssignable(charSequenceType, objectType, charSequenceType, stringType).equalTo(false, true, true);
 		assertAssignable(stringType, objectType, charSequenceType, stringType).equalTo(false, false, true);
+
+		assertTrue(objectType.isAssignableFrom(String.class));
+		assertTrue(objectType.isAssignableFrom(StringBuilder.class));
+		assertTrue(charSequenceType.isAssignableFrom(String.class));
+		assertTrue(charSequenceType.isAssignableFrom(StringBuilder.class));
+		assertTrue(stringType.isAssignableFrom(String.class));
+		assertFalse(stringType.isAssignableFrom(StringBuilder.class));
+
+		assertTrue(objectType.isInstance("a String"));
+		assertTrue(objectType.isInstance(new StringBuilder("a StringBuilder")));
+		assertTrue(charSequenceType.isInstance("a String"));
+		assertTrue(charSequenceType.isInstance(new StringBuilder("a StringBuilder")));
+		assertTrue(stringType.isInstance("a String"));
+		assertFalse(stringType.isInstance(new StringBuilder("a StringBuilder")));
 	}
 
 	@Test
@@ -1017,7 +1085,6 @@ public class ResolvableTypeTests {
 
 	@Test
 	public void isAssignableFromForWildcards() throws Exception {
-
 		ResolvableType object = ResolvableType.forClass(Object.class);
 		ResolvableType charSequence = ResolvableType.forClass(CharSequence.class);
 		ResolvableType string = ResolvableType.forClass(String.class);
@@ -1210,6 +1277,25 @@ public class ResolvableTypeTests {
 		assertEquals("java.util.Collection<org.springframework.core.ResolvableTypeTests$IBase<?>>", type.toString());
 	}
 
+	@Test
+	public void testSpr12701() throws Exception {
+		ResolvableType resolvableType = ResolvableType.forClassWithGenerics(Callable.class, String.class);
+		Type type = resolvableType.getType();
+		assertThat(type, is(instanceOf(ParameterizedType.class)));
+		assertThat(((ParameterizedType) type).getRawType(), is(equalTo(Callable.class)));
+		assertThat(((ParameterizedType) type).getActualTypeArguments().length, is(equalTo(1)));
+		assertThat(((ParameterizedType) type).getActualTypeArguments()[0], is(equalTo(String.class)));
+	}
+
+	@Test
+	public void testSpr14648() throws Exception {
+		ResolvableType collectionClass = ResolvableType.forRawClass(Collection.class);
+		ResolvableType setClass = ResolvableType.forRawClass(Set.class);
+		ResolvableType fromReturnType = ResolvableType.forMethodReturnType(Methods.class.getMethod("wildcardSet"));
+		assertTrue(collectionClass.isAssignableFrom(fromReturnType));
+		assertTrue(setClass.isAssignableFrom(fromReturnType));
+	}
+
 
 	private ResolvableType testSerialization(ResolvableType type) throws Exception {
 		ByteArrayOutputStream bos = new ByteArrayOutputStream();
@@ -1308,7 +1394,7 @@ public class ResolvableTypeTests {
 	}
 
 
-	static interface Methods<T> {
+	interface Methods<T> {
 
 		List<CharSequence> charSequenceReturn();
 
@@ -1321,6 +1407,8 @@ public class ResolvableTypeTests {
 		void typedParameter(T p);
 
 		T typedReturn();
+
+		Set<?> wildcardSet();
 	}
 
 
@@ -1376,7 +1464,7 @@ public class ResolvableTypeTests {
 	}
 
 
-	static interface TypedMethods extends Methods<String> {
+	interface TypedMethods extends Methods<String> {
 	}
 
 
@@ -1405,6 +1493,23 @@ public class ResolvableTypeTests {
 	public interface MyInterfaceType<T> {
 	}
 
+	public class MyGenericInterfaceType<T> implements MyInterfaceType<T>, ResolvableTypeProvider {
+
+		private final Class<T> type;
+
+		public MyGenericInterfaceType(Class<T> type) {
+			this.type = type;
+		}
+
+		@Override
+		public ResolvableType getResolvableType() {
+			if (this.type == null) {
+				return null;
+			}
+			return ResolvableType.forClassWithGenerics(getClass(), this.type);
+		}
+	}
+
 
 	public class MySimpleInterfaceType implements MyInterfaceType<String> {
 	}
@@ -1432,19 +1537,19 @@ public class ResolvableTypeTests {
 	}
 
 
-	static interface Wildcard<T extends Number> extends List<T> {
+	interface Wildcard<T extends Number> extends List<T> {
 	}
 
 
-	static interface RawExtendsWildcard extends Wildcard {
+	interface RawExtendsWildcard extends Wildcard {
 	}
 
 
-	static interface VariableNameSwitch<V, K> extends MultiValueMap<K, V> {
+	interface VariableNameSwitch<V, K> extends MultiValueMap<K, V> {
 	}
 
 
-	static interface ListOfGenericArray extends List<List<String>[]> {
+	interface ListOfGenericArray extends List<List<String>[]> {
 	}
 
 
