@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,12 +20,14 @@ import java.awt.Color;
 import java.lang.reflect.Method;
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.nio.charset.Charset;
 import java.time.ZoneId;
 import java.util.AbstractList;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Currency;
 import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
@@ -37,6 +39,7 @@ import java.util.Map;
 import java.util.Optional;
 import java.util.Properties;
 import java.util.Set;
+import java.util.TimeZone;
 import java.util.UUID;
 import java.util.stream.Stream;
 
@@ -240,6 +243,26 @@ public class DefaultConversionServiceTests {
 	public void testEnumToString() {
 		assertEquals("BAR", conversionService.convert(Foo.BAR, String.class));
 	}
+	
+	@Test
+	public void testIntegerToEnum() throws Exception {
+		assertEquals(Foo.BAR, conversionService.convert(0, Foo.class));
+	}
+	
+	@Test
+	public void testIntegerToEnumWithSubclass() throws Exception {
+		assertEquals(SubFoo.BAZ, conversionService.convert(1, SubFoo.BAR.getClass()));
+	}
+	
+	@Test
+	public void testIntegerToEnumNull() {
+		assertEquals(null, conversionService.convert(null, Foo.class));
+	}
+	
+	@Test
+	public void testEnumToInteger() {
+		assertEquals(Integer.valueOf(0), conversionService.convert(Foo.BAR, Integer.class));
+	}
 
 	@Test
 	public void testStringToEnumSet() throws Exception {
@@ -252,6 +275,26 @@ public class DefaultConversionServiceTests {
 		assertEquals(Locale.ENGLISH, conversionService.convert("en", Locale.class));
 	}
 
+	@Test
+	public void testStringToCharset() {
+		assertEquals(Charset.forName("UTF-8"), conversionService.convert("UTF-8", Charset.class));
+	}
+
+	@Test
+	public void testCharsetToString() {
+		assertEquals("UTF-8", conversionService.convert(Charset.forName("UTF-8"), String.class));
+	}
+
+	@Test
+	public void testStringToCurrency() {
+		assertEquals(Currency.getInstance("EUR"), conversionService.convert("EUR", Currency.class));
+	}
+
+	@Test
+	public void testCurrencyToString() {
+		assertEquals("USD", conversionService.convert(Currency.getInstance("USD"), String.class));
+	}
+
 	@Test
 	public void testStringToString() {
 		String str = "test";
@@ -290,7 +333,7 @@ public class DefaultConversionServiceTests {
 
 	@Test
 	public void convertArrayToCollectionInterface() {
-		List<?> result = conversionService.convert(new String[] { "1", "2", "3" }, List.class);
+		List<?> result = conversionService.convert(new String[] {"1", "2", "3"}, List.class);
 		assertEquals("1", result.get(0));
 		assertEquals("2", result.get(1));
 		assertEquals("3", result.get(2));
@@ -299,19 +342,29 @@ public class DefaultConversionServiceTests {
 	@Test
 	public void convertArrayToCollectionGenericTypeConversion() throws Exception {
 		@SuppressWarnings("unchecked")
-		List<Integer> result = (List<Integer>) conversionService.convert(new String[] { "1", "2", "3" }, TypeDescriptor
+		List<Integer> result = (List<Integer>) conversionService.convert(new String[] {"1", "2", "3"}, TypeDescriptor
 				.valueOf(String[].class), new TypeDescriptor(getClass().getDeclaredField("genericList")));
 		assertEquals(new Integer("1"), result.get(0));
 		assertEquals(new Integer("2"), result.get(1));
 		assertEquals(new Integer("3"), result.get(2));
 	}
 
+	@Test
+	public void convertArrayToStream() throws Exception {
+		String[] source = {"1", "3", "4"};
+		@SuppressWarnings("unchecked")
+		Stream<Integer> result = (Stream<Integer>) this.conversionService.convert(source,
+				TypeDescriptor.valueOf(String[].class),
+				new TypeDescriptor(getClass().getDeclaredField("genericStream")));
+		assertEquals(8, result.mapToInt((x) -> x).sum());
+	}
+
 	@Test
 	public void testSpr7766() throws Exception {
 		ConverterRegistry registry = (conversionService);
 		registry.addConverter(new ColorConverter());
 		@SuppressWarnings("unchecked")
-		List<Color> colors = (List<Color>) conversionService.convert(new String[] { "ffffff", "#000000" },
+		List<Color> colors = (List<Color>) conversionService.convert(new String[] {"ffffff", "#000000"},
 				TypeDescriptor.valueOf(String[].class),
 				new TypeDescriptor(new MethodParameter(getClass().getMethod("handlerMethod", List.class), 0)));
 		assertEquals(2, colors.size());
@@ -321,7 +374,7 @@ public class DefaultConversionServiceTests {
 
 	@Test
 	public void convertArrayToCollectionImpl() {
-		LinkedList<?> result = conversionService.convert(new String[] { "1", "2", "3" }, LinkedList.class);
+		LinkedList<?> result = conversionService.convert(new String[] {"1", "2", "3"}, LinkedList.class);
 		assertEquals("1", result.get(0));
 		assertEquals("2", result.get(1));
 		assertEquals("3", result.get(2));
@@ -329,7 +382,7 @@ public class DefaultConversionServiceTests {
 
 	@Test(expected = ConversionFailedException.class)
 	public void convertArrayToAbstractCollection() {
-		conversionService.convert(new String[] { "1", "2", "3" }, AbstractList.class);
+		conversionService.convert(new String[]{"1", "2", "3"}, AbstractList.class);
 	}
 
 	public static enum FooEnum {
@@ -338,13 +391,13 @@ public class DefaultConversionServiceTests {
 
 	@Test
 	public void convertArrayToString() {
-		String result = conversionService.convert(new String[] { "1", "2", "3" }, String.class);
+		String result = conversionService.convert(new String[] {"1", "2", "3"}, String.class);
 		assertEquals("1,2,3", result);
 	}
 
 	@Test
 	public void convertArrayToStringWithElementConversion() {
-		String result = conversionService.convert(new Integer[] { 1, 2, 3 }, String.class);
+		String result = conversionService.convert(new Integer[] {1, 2, 3}, String.class);
 		assertEquals("1,2,3", result);
 	}
 
@@ -389,21 +442,21 @@ public class DefaultConversionServiceTests {
 
 	@Test
 	public void convertArrayToObject() {
-		Object[] array = new Object[] { 3L };
+		Object[] array = new Object[] {3L};
 		Object result = conversionService.convert(array, Long.class);
 		assertEquals(3L, result);
 	}
 
 	@Test
 	public void convertArrayToObjectWithElementConversion() {
-		String[] array = new String[] { "3" };
+		String[] array = new String[] {"3"};
 		Integer result = conversionService.convert(array, Integer.class);
 		assertEquals(new Integer(3), result);
 	}
 
 	@Test
 	public void convertArrayToObjectAssignableTargetType() {
-		Long[] array = new Long[] { 3L };
+		Long[] array = new Long[] {3L};
 		Long[] result = (Long[]) conversionService.convert(array, Object.class);
 		assertArrayEquals(array, result);
 	}
@@ -528,9 +581,9 @@ public class DefaultConversionServiceTests {
 	}
 
 	@Test
-	@SuppressWarnings("unchecked")
+	@SuppressWarnings("rawtypes")
 	public void convertObjectToCollection() {
-		List<String> result = (List<String>) conversionService.convert(3L, List.class);
+		List result = conversionService.convert(3L, List.class);
 		assertEquals(1, result.size());
 		assertEquals(3L, result.get(0));
 	}
@@ -564,7 +617,7 @@ public class DefaultConversionServiceTests {
 	public void convertArrayToWrapperArray() {
 		byte[] byteArray = new byte[] {1, 2, 3};
 		Byte[] converted = conversionService.convert(byteArray, Byte[].class);
-		assertThat(converted, equalTo(new Byte[] {1, 2, 3}));
+		assertThat(converted, equalTo(new Byte[]{1, 2, 3}));
 	}
 
 	@Test
@@ -751,6 +804,11 @@ public class DefaultConversionServiceTests {
 		assertEquals("toString() invocations", 0, SSN.toStringCount);
 	}
 
+	@Test
+	public void convertStringToTimezone() {
+		assertEquals("GMT+02:00", conversionService.convert("GMT+2", TimeZone.class).getID());
+	}
+
 	@Test
 	public void convertObjectToStringWithJavaTimeOfMethodPresent() {
 		assertTrue(conversionService.convert(ZoneId.of("GMT+1"), String.class).startsWith("GMT+"));
@@ -799,7 +857,7 @@ public class DefaultConversionServiceTests {
 	@Test
 	public void convertStringToCharArray() throws Exception {
 		char[] converted = conversionService.convert("a,b,c", char[].class);
-		assertThat(converted, equalTo(new char[] {'a', 'b', 'c'}));
+		assertThat(converted, equalTo(new char[]{'a', 'b', 'c'}));
 	}
 
 	@Test
@@ -811,14 +869,14 @@ public class DefaultConversionServiceTests {
 			}
 		});
 		char[] converted = conversionService.convert("abc", char[].class);
-		assertThat(converted, equalTo(new char[] { 'a', 'b', 'c' }));
+		assertThat(converted, equalTo(new char[] {'a', 'b', 'c'}));
 	}
 
 	@Test
 	@SuppressWarnings("unchecked")
 	public void multidimensionalArrayToListConversionShouldConvertEntriesCorrectly() {
-		String[][] grid = new String[][] { new String[] { "1", "2", "3", "4" }, new String[] { "5", "6", "7", "8" },
-				new String[] { "9", "10", "11", "12" } };
+		String[][] grid = new String[][] {new String[] {"1", "2", "3", "4"}, new String[] {"5", "6", "7", "8"},
+				new String[] {"9", "10", "11", "12"}};
 		List<String[]> converted = conversionService.convert(grid, List.class);
 		String[][] convertedBack = conversionService.convert(converted, String[][].class);
 		assertArrayEquals(grid, convertedBack);
@@ -827,16 +885,15 @@ public class DefaultConversionServiceTests {
 	@Test
 	public void convertCannotOptimizeArray() {
 		conversionService.addConverter(new Converter<Byte, Byte>() {
-
 			@Override
 			public Byte convert(Byte source) {
 				return (byte) (source + 1);
 			}
 		});
-		byte[] byteArray = new byte[] { 1, 2, 3 };
+		byte[] byteArray = new byte[] {1, 2, 3};
 		byte[] converted = conversionService.convert(byteArray, byte[].class);
 		assertNotSame(byteArray, converted);
-		assertTrue(Arrays.equals(new byte[] { 2, 3, 4 }, converted));
+		assertTrue(Arrays.equals(new byte[] {2, 3, 4}, converted));
 	}
 
 	@Test
@@ -886,6 +943,8 @@ public class DefaultConversionServiceTests {
 
 	public List<Integer> genericList = new ArrayList<Integer>();
 
+	public Stream<Integer> genericStream;
+
 	public Map<Integer, FooEnum> genericMap = new HashMap<Integer, FooEnum>();
 
 	public EnumSet<Foo> enumSet;
@@ -898,6 +957,7 @@ public class DefaultConversionServiceTests {
 
 
 	public enum Foo {
+
 		BAR, BAZ
 	}
 
@@ -924,7 +984,12 @@ public class DefaultConversionServiceTests {
 	public class ColorConverter implements Converter<String, Color> {
 
 		@Override
-		public Color convert(String source) { if (!source.startsWith("#")) source = "#" + source; return Color.decode(source); }
+		public Color convert(String source) {
+			if (!source.startsWith("#")) {
+				source = "#" + source;
+			}
+			return Color.decode(source);
+		}
 	}
 
 
@@ -991,8 +1056,8 @@ public class DefaultConversionServiceTests {
 	private static class SSN {
 
 		static int constructorCount = 0;
-		static int toStringCount = 0;
 
+		static int toStringCount = 0;
 
 		static void reset() {
 			constructorCount = 0;
