@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2013 the original author or authors.
+ * Copyright 2002-2015 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,15 +18,19 @@ package org.springframework.util.xml;
 
 import java.io.InputStream;
 import java.io.StringReader;
+
 import javax.xml.stream.XMLEventReader;
 import javax.xml.stream.XMLInputFactory;
 import javax.xml.stream.XMLStreamException;
 
+import org.junit.Test;
+
+import org.xml.sax.Attributes;
 import org.xml.sax.ContentHandler;
 import org.xml.sax.InputSource;
-import org.xml.sax.helpers.AttributesImpl;
 
-import static org.mockito.BDDMockito.*;
+import static org.mockito.Matchers.*;
+import static org.mockito.Mockito.*;
 
 public class StaxEventXMLReaderTests extends AbstractStaxXMLReaderTestCase {
 
@@ -37,7 +41,8 @@ public class StaxEventXMLReaderTests extends AbstractStaxXMLReaderTestCase {
 		return new StaxEventXMLReader(inputFactory.createXMLEventReader(inputStream));
 	}
 
-	public void testPartial() throws Exception {
+	@Test
+	public void partial() throws Exception {
 		XMLInputFactory inputFactory = XMLInputFactory.newInstance();
 		XMLEventReader eventReader = inputFactory.createXMLEventReader(new StringReader(CONTENT));
 		eventReader.nextTag(); // skip to root
@@ -46,7 +51,7 @@ public class StaxEventXMLReaderTests extends AbstractStaxXMLReaderTestCase {
 		xmlReader.setContentHandler(contentHandler);
 		xmlReader.parse(new InputSource());
 		verify(contentHandler).startDocument();
-		verify(contentHandler).startElement("http://springframework.org/spring-ws", "child", "child", new AttributesImpl());
+		verify(contentHandler).startElement(eq("http://springframework.org/spring-ws"), eq("child"), eq("child"), any(Attributes.class));
 		verify(contentHandler).endElement("http://springframework.org/spring-ws", "child", "child");
 		verify(contentHandler).endDocument();
 	}
