@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,13 +16,19 @@
 
 package org.springframework.web.client;
 
+import java.io.IOException;
 import java.net.URI;
+import java.net.URISyntaxException;
 import java.nio.charset.Charset;
+import java.util.Collections;
 import java.util.EnumSet;
 import java.util.Set;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
+import java.util.concurrent.TimeUnit;
 
-import org.junit.Before;
+import org.junit.Assert;
 import org.junit.Test;
 
 import org.springframework.core.io.ClassPathResource;
@@ -30,10 +36,15 @@ import org.springframework.core.io.Resource;
 import org.springframework.http.HttpEntity;
 import org.springframework.http.HttpHeaders;
 import org.springframework.http.HttpMethod;
+import org.springframework.http.HttpRequest;
 import org.springframework.http.HttpStatus;
 import org.springframework.http.MediaType;
 import org.springframework.http.ResponseEntity;
+import org.springframework.http.client.AsyncClientHttpRequestExecution;
+import org.springframework.http.client.AsyncClientHttpRequestInterceptor;
+import org.springframework.http.client.ClientHttpResponse;
 import org.springframework.http.client.HttpComponentsAsyncClientHttpRequestFactory;
+import org.springframework.http.client.support.HttpRequestWrapper;
 import org.springframework.util.LinkedMultiValueMap;
 import org.springframework.util.MultiValueMap;
 import org.springframework.util.concurrent.ListenableFuture;
@@ -47,20 +58,14 @@ import static org.junit.Assert.*;
  */
 public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCase {
 
-	private AsyncRestTemplate template;
-
-
-	@Before
-	public void createTemplate() {
-		template = new AsyncRestTemplate(new HttpComponentsAsyncClientHttpRequestFactory());
-	}
+	private final AsyncRestTemplate template = new AsyncRestTemplate(
+			new HttpComponentsAsyncClientHttpRequestFactory());
 
 
 	@Test
 	public void getEntity() throws Exception {
-		Future<ResponseEntity<String>> futureEntity =
-				template.getForEntity(baseUrl + "/{method}", String.class, "get");
-		ResponseEntity<String> entity = futureEntity.get();
+		Future<ResponseEntity<String>> future = template.getForEntity(baseUrl + "/{method}", String.class, "get");
+		ResponseEntity<String> entity = future.get();
 		assertEquals("Invalid content", helloWorld, entity.getBody());
 		assertFalse("No headers", entity.getHeaders().isEmpty());
 		assertEquals("Invalid content-type", textContentType, entity.getHeaders().getContentType());
@@ -69,10 +74,9 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 
 	@Test
 	public void multipleFutureGets() throws Exception {
-		Future<ResponseEntity<String>> futureEntity =
-				template.getForEntity(baseUrl + "/{method}", String.class, "get");
-		futureEntity.get();
-		futureEntity.get();
+		Future<ResponseEntity<String>> future = template.getForEntity(baseUrl + "/{method}", String.class, "get");
+		future.get();
+		future.get();
 	}
 
 	@Test
@@ -92,9 +96,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		// wait till done
-		while (!futureEntity.isDone()) {
-		}
+		waitTillDone(futureEntity);
 	}
 
 	@Test
@@ -107,9 +109,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 			assertEquals("Invalid content-type", textContentType, entity.getHeaders().getContentType());
 			assertEquals("Invalid status code", HttpStatus.OK, entity.getStatusCode());
 		}, ex -> fail(ex.getMessage()));
-		// wait till done
-		while (!futureEntity.isDone()) {
-		}
+		waitTillDone(futureEntity);
 	}
 
 	@Test
@@ -164,8 +164,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		while (!headersFuture.isDone()) {
-		}
+		waitTillDone(headersFuture);
 	}
 
 	@Test
@@ -173,15 +172,14 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 		ListenableFuture<HttpHeaders> headersFuture = template.headForHeaders(baseUrl + "/get");
 		headersFuture.addCallback(result -> assertTrue("No Content-Type header",
 				result.containsKey("Content-Type")), ex -> fail(ex.getMessage()));
-		while (!headersFuture.isDone()) {
-		}
+		waitTillDone(headersFuture);
 	}
 
 	@Test
 	public void postForLocation() throws Exception  {
 		HttpHeaders entityHeaders = new HttpHeaders();
-		entityHeaders.setContentType(new MediaType("text", "plain", Charset.forName("ISO-8859-15")));
-		HttpEntity<String> entity = new HttpEntity<String>(helloWorld, entityHeaders);
+		entityHeaders.setContentType(new MediaType("text", "plain", Charset.forName("ISO-8859-1")));
+		HttpEntity<String> entity = new HttpEntity<>(helloWorld, entityHeaders);
 		Future<URI> locationFuture = template.postForLocation(baseUrl + "/{method}", entity, "post");
 		URI location = locationFuture.get();
 		assertEquals("Invalid location", new URI(baseUrl + "/post/1"), location);
@@ -190,8 +188,8 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 	@Test
 	public void postForLocationCallback() throws Exception  {
 		HttpHeaders entityHeaders = new HttpHeaders();
-		entityHeaders.setContentType(new MediaType("text", "plain", Charset.forName("ISO-8859-15")));
-		HttpEntity<String> entity = new HttpEntity<String>(helloWorld, entityHeaders);
+		entityHeaders.setContentType(new MediaType("text", "plain", Charset.forName("ISO-8859-1")));
+		HttpEntity<String> entity = new HttpEntity<>(helloWorld, entityHeaders);
 		final URI expected = new URI(baseUrl + "/post/1");
 		ListenableFuture<URI> locationFuture = template.postForLocation(baseUrl + "/{method}", entity, "post");
 		locationFuture.addCallback(new ListenableFutureCallback<URI>() {
@@ -204,21 +202,19 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		while (!locationFuture.isDone()) {
-		}
+		waitTillDone(locationFuture);
 	}
 
 	@Test
 	public void postForLocationCallbackWithLambdas() throws Exception  {
 		HttpHeaders entityHeaders = new HttpHeaders();
-		entityHeaders.setContentType(new MediaType("text", "plain", Charset.forName("ISO-8859-15")));
-		HttpEntity<String> entity = new HttpEntity<String>(helloWorld, entityHeaders);
+		entityHeaders.setContentType(new MediaType("text", "plain", Charset.forName("ISO-8859-1")));
+		HttpEntity<String> entity = new HttpEntity<>(helloWorld, entityHeaders);
 		final URI expected = new URI(baseUrl + "/post/1");
 		ListenableFuture<URI> locationFuture = template.postForLocation(baseUrl + "/{method}", entity, "post");
 		locationFuture.addCallback(result -> assertEquals("Invalid location", expected, result),
 				ex -> fail(ex.getMessage()));
-		while (!locationFuture.isDone()) {
-		}
+		waitTillDone(locationFuture);
 	}
 
 	@Test
@@ -245,8 +241,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		while (!responseEntityFuture.isDone()) {
-		}
+		waitTillDone(responseEntityFuture);
 	}
 
 	@Test
@@ -254,10 +249,10 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 		HttpEntity<String> requestEntity = new HttpEntity<>(helloWorld);
 		ListenableFuture<ResponseEntity<String>> responseEntityFuture =
 				template.postForEntity(baseUrl + "/{method}", requestEntity, String.class, "post");
-		responseEntityFuture.addCallback(result -> assertEquals("Invalid content", helloWorld, result.getBody()),
+		responseEntityFuture.addCallback(
+				result -> assertEquals("Invalid content", helloWorld, result.getBody()),
 				ex -> fail(ex.getMessage()));
-		while (!responseEntityFuture.isDone()) {
-		}
+		waitTillDone(responseEntityFuture);
 	}
 
 	@Test
@@ -281,8 +276,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		while (!responseEntityFuture.isDone()) {
-		}
+		waitTillDone(responseEntityFuture);
 	}
 
 	@Test
@@ -304,29 +298,61 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		while (!deletedFuture.isDone()) {
-		}
+		waitTillDone(deletedFuture);
 	}
 
 	@Test
 	public void deleteCallbackWithLambdas() throws Exception  {
 		ListenableFuture<?> deletedFuture = template.delete(new URI(baseUrl + "/delete"));
-		deletedFuture.addCallback(result -> assertNull(result), ex -> fail(ex.getMessage()));
-		while (!deletedFuture.isDone()) {
+		deletedFuture.addCallback(Assert::assertNull, ex -> fail(ex.getMessage()));
+		waitTillDone(deletedFuture);
+	}
+
+	@Test
+	public void identicalExceptionThroughGetAndCallback() throws Exception {
+		final HttpClientErrorException[] callbackException = new HttpClientErrorException[1];
+
+		final CountDownLatch latch = new CountDownLatch(1);
+		ListenableFuture<?> future = template.execute(baseUrl + "/status/notfound", HttpMethod.GET, null, null);
+		future.addCallback(new ListenableFutureCallback<Object>() {
+			@Override
+			public void onSuccess(Object result) {
+				fail("onSuccess not expected");
+			}
+			@Override
+			public void onFailure(Throwable ex) {
+				assertTrue(ex instanceof HttpClientErrorException);
+				callbackException[0] = (HttpClientErrorException) ex;
+				latch.countDown();
+			}
+		});
+
+		try {
+			future.get();
+			fail("Exception expected");
+		}
+		catch (ExecutionException ex) {
+			Throwable cause = ex.getCause();
+			assertTrue(cause instanceof HttpClientErrorException);
+			latch.await(5, TimeUnit.SECONDS);
+			assertSame(callbackException[0], cause);
 		}
 	}
 
 	@Test
-	public void notFound() throws Exception {
+	public void notFoundGet() throws Exception {
 		try {
 			Future<?> future = template.execute(baseUrl + "/status/notfound", HttpMethod.GET, null, null);
 			future.get();
 			fail("HttpClientErrorException expected");
 		}
-		catch (HttpClientErrorException ex) {
-			assertEquals(HttpStatus.NOT_FOUND, ex.getStatusCode());
-			assertNotNull(ex.getStatusText());
-			assertNotNull(ex.getResponseBodyAsString());
+		catch (ExecutionException ex) {
+			assertTrue(ex.getCause() instanceof HttpClientErrorException);
+			HttpClientErrorException cause = (HttpClientErrorException)ex.getCause();
+
+			assertEquals(HttpStatus.NOT_FOUND, cause.getStatusCode());
+			assertNotNull(cause.getStatusText());
+			assertNotNull(cause.getResponseBodyAsString());
 		}
 	}
 
@@ -347,8 +373,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				assertNotNull(ex.getResponseBodyAsString());
 			}
 		});
-		while (!future.isDone()) {
-		}
+		waitTillDone(future);
 	}
 
 	@Test
@@ -361,8 +386,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				assertNotNull(hcex.getStatusText());
 				assertNotNull(hcex.getResponseBodyAsString());
 		});
-		while (!future.isDone()) {
-		}
+		waitTillDone(future);
 	}
 
 	@Test
@@ -372,10 +396,13 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 			future.get();
 			fail("HttpServerErrorException expected");
 		}
-		catch (HttpServerErrorException ex) {
-			assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, ex.getStatusCode());
-			assertNotNull(ex.getStatusText());
-			assertNotNull(ex.getResponseBodyAsString());
+		catch (ExecutionException ex) {
+			assertTrue(ex.getCause() instanceof HttpServerErrorException);
+			HttpServerErrorException cause = (HttpServerErrorException)ex.getCause();
+
+			assertEquals(HttpStatus.INTERNAL_SERVER_ERROR, cause.getStatusCode());
+			assertNotNull(cause.getStatusText());
+			assertNotNull(cause.getResponseBodyAsString());
 		}
 	}
 
@@ -396,8 +423,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				assertNotNull(hsex.getResponseBodyAsString());
 			}
 		});
-		while (!future.isDone()) {
-		}
+		waitTillDone(future);
 	}
 
 	@Test
@@ -410,8 +436,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				assertNotNull(hsex.getStatusText());
 				assertNotNull(hsex.getResponseBodyAsString());
 		});
-		while (!future.isDone()) {
-		}
+		waitTillDone(future);
 	}
 
 	@Test
@@ -436,8 +461,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		while (!allowedFuture.isDone()) {
-		}
+		waitTillDone(allowedFuture);
 	}
 
 	@Test
@@ -446,8 +470,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 		allowedFuture.addCallback(result -> assertEquals("Invalid response",
 				EnumSet.of(HttpMethod.GET, HttpMethod.OPTIONS, HttpMethod.HEAD,HttpMethod.TRACE), result),
 				ex -> fail(ex.getMessage()));
-		while (!allowedFuture.isDone()) {
-		}
+		waitTillDone(allowedFuture);
 	}
 
 	@Test
@@ -480,8 +503,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		while (!responseFuture.isDone()) {
-		}
+		waitTillDone(responseFuture);
 	}
 
 	@Test
@@ -494,8 +516,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				template.exchange(baseUrl + "/{method}", HttpMethod.GET, requestEntity, String.class, "get");
 		responseFuture.addCallback(result -> assertEquals("Invalid content", helloWorld,
 				result.getBody()), ex -> fail(ex.getMessage()));
-		while (!responseFuture.isDone()) {
-		}
+		waitTillDone(responseFuture);
 	}
 
 	@Test
@@ -503,7 +524,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 		HttpHeaders requestHeaders = new HttpHeaders();
 		requestHeaders.set("MyHeader", "MyValue");
 		requestHeaders.setContentType(MediaType.TEXT_PLAIN);
-		HttpEntity<String> requestEntity = new HttpEntity<String>(helloWorld, requestHeaders);
+		HttpEntity<String> requestEntity = new HttpEntity<>(helloWorld, requestHeaders);
 		Future<ResponseEntity<Void>> resultFuture =
 				template.exchange(baseUrl + "/{method}", HttpMethod.POST, requestEntity, Void.class, "post");
 		ResponseEntity<Void> result = resultFuture.get();
@@ -517,7 +538,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 		HttpHeaders requestHeaders = new HttpHeaders();
 		requestHeaders.set("MyHeader", "MyValue");
 		requestHeaders.setContentType(MediaType.TEXT_PLAIN);
-		HttpEntity<String> requestEntity = new HttpEntity<String>(helloWorld, requestHeaders);
+		HttpEntity<String> requestEntity = new HttpEntity<>(helloWorld, requestHeaders);
 		ListenableFuture<ResponseEntity<Void>> resultFuture =
 				template.exchange(baseUrl + "/{method}", HttpMethod.POST, requestEntity, Void.class, "post");
 		final URI expected =new URI(baseUrl + "/post/1");
@@ -532,8 +553,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 				fail(ex.getMessage());
 			}
 		});
-		while (!resultFuture.isDone()) {
-		}
+		waitTillDone(resultFuture);
 	}
 
 	@Test
@@ -541,7 +561,7 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 		HttpHeaders requestHeaders = new HttpHeaders();
 		requestHeaders.set("MyHeader", "MyValue");
 		requestHeaders.setContentType(MediaType.TEXT_PLAIN);
-		HttpEntity<String> requestEntity = new HttpEntity<String>(helloWorld, requestHeaders);
+		HttpEntity<String> requestEntity = new HttpEntity<>(helloWorld, requestHeaders);
 		ListenableFuture<ResponseEntity<Void>> resultFuture =
 				template.exchange(baseUrl + "/{method}", HttpMethod.POST, requestEntity, Void.class, "post");
 		final URI expected =new URI(baseUrl + "/post/1");
@@ -549,13 +569,12 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 			assertEquals("Invalid location", expected, result.getHeaders().getLocation());
 			assertFalse(result.hasBody());
 			}, ex -> fail(ex.getMessage()));
-		while (!resultFuture.isDone()) {
-		}
+		waitTillDone(resultFuture);
 	}
 
 	@Test
 	public void multipart() throws Exception {
-		MultiValueMap<String, Object> parts = new LinkedMultiValueMap<String, Object>();
+		MultiValueMap<String, Object> parts = new LinkedMultiValueMap<>();
 		parts.add("name 1", "value 1");
 		parts.add("name 2", "value 2+1");
 		parts.add("name 2", "value 2+2");
@@ -567,4 +586,72 @@ public class AsyncRestTemplateIntegrationTests extends AbstractJettyServerTestCa
 		future.get();
 	}
 
+	@Test
+	public void getAndInterceptResponse() throws Exception {
+		RequestInterceptor interceptor = new RequestInterceptor();
+		template.setInterceptors(Collections.singletonList(interceptor));
+		ListenableFuture<ResponseEntity<String>> future = template.getForEntity("/get", String.class);
+
+		interceptor.latch.await(5, TimeUnit.SECONDS);
+		assertNotNull(interceptor.response);
+		assertEquals(HttpStatus.OK, interceptor.response.getStatusCode());
+		assertNull(interceptor.exception);
+		assertEquals(helloWorld, future.get().getBody());
+	}
+
+	@Test
+	public void getAndInterceptError() throws Exception {
+		RequestInterceptor interceptor = new RequestInterceptor();
+		template.setInterceptors(Collections.singletonList(interceptor));
+		template.getForEntity("/status/notfound", String.class);
+
+		interceptor.latch.await(5, TimeUnit.SECONDS);
+		assertNotNull(interceptor.response);
+		assertEquals(HttpStatus.NOT_FOUND, interceptor.response.getStatusCode());
+		assertNull(interceptor.exception);
+	}
+
+	private void waitTillDone(ListenableFuture<?> future) {
+		while (!future.isDone()) {
+		}
+	}
+
+
+	private static class RequestInterceptor implements AsyncClientHttpRequestInterceptor {
+
+		private final CountDownLatch latch = new CountDownLatch(1);
+
+		private volatile ClientHttpResponse response;
+
+		private volatile Throwable exception;
+
+		@Override
+		public ListenableFuture<ClientHttpResponse> intercept(HttpRequest request, byte[] body,
+				AsyncClientHttpRequestExecution execution) throws IOException {
+
+			request = new HttpRequestWrapper(request) {
+				@Override
+				public URI getURI() {
+					try {
+						return new URI(baseUrl + super.getURI().toString());
+					}
+					catch (URISyntaxException ex) {
+						throw new IllegalStateException(ex);
+					}
+				}
+			};
+
+			ListenableFuture<ClientHttpResponse> future = execution.executeAsync(request, body);
+			future.addCallback(
+					resp -> {
+						response = resp;
+						this.latch.countDown();
+					},
+					ex -> {
+						exception = ex;
+						this.latch.countDown();
+					});
+			return future;
+		}
+	}
 }
