@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,13 +20,17 @@ import java.io.UnsupportedEncodingException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.nio.charset.Charset;
+import java.util.ArrayList;
 import java.util.EnumSet;
+import java.util.List;
 import java.util.Set;
 
+import com.fasterxml.jackson.annotation.JsonTypeInfo;
+import com.fasterxml.jackson.annotation.JsonTypeName;
 import com.fasterxml.jackson.annotation.JsonView;
-import org.junit.Before;
 import org.junit.Test;
 
+import org.springframework.core.ParameterizedTypeReference;
 import org.springframework.core.io.ClassPathResource;
 import org.springframework.core.io.Resource;
 import org.springframework.http.HttpEntity;
@@ -34,6 +38,7 @@ import org.springframework.http.HttpHeaders;
 import org.springframework.http.HttpMethod;
 import org.springframework.http.HttpStatus;
 import org.springframework.http.MediaType;
+import org.springframework.http.RequestEntity;
 import org.springframework.http.ResponseEntity;
 import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
 import org.springframework.http.converter.json.MappingJacksonValue;
@@ -47,12 +52,8 @@ import static org.junit.Assert.*;
  */
 public class RestTemplateIntegrationTests extends AbstractJettyServerTestCase {
 
-	private RestTemplate template;
+	private final RestTemplate template = new RestTemplate(new HttpComponentsClientHttpRequestFactory());
 
-	@Before
-	public void createTemplate() {
-		template = new RestTemplate(new HttpComponentsClientHttpRequestFactory());
-	}
 
 	@Test
 	public void getString() {
@@ -179,6 +180,16 @@ public class RestTemplateIntegrationTests extends AbstractJettyServerTestCase {
 		template.postForLocation(baseUrl + "/multipart", parts);
 	}
 
+	@Test
+	public void form() throws UnsupportedEncodingException {
+		MultiValueMap<String, String> form = new LinkedMultiValueMap<String, String>();
+		form.add("name 1", "value 1");
+		form.add("name 2", "value 2+1");
+		form.add("name 2", "value 2+2");
+
+		template.postForLocation(baseUrl + "/form", form);
+	}
+
 	@Test
 	public void exchangeGet() throws Exception {
 		HttpHeaders requestHeaders = new HttpHeaders();
@@ -209,7 +220,7 @@ public class RestTemplateIntegrationTests extends AbstractJettyServerTestCase {
 		bean.setWith2("with");
 		bean.setWithout("without");
 		HttpEntity<MySampleBean> entity = new HttpEntity<MySampleBean>(bean, entityHeaders);
-		String s = template.postForObject(baseUrl + "/jsonpost", entity, String.class, "post");
+		String s = template.postForObject(baseUrl + "/jsonpost", entity, String.class);
 		assertTrue(s.contains("\"with1\":\"with\""));
 		assertTrue(s.contains("\"with2\":\"with\""));
 		assertTrue(s.contains("\"without\":\"without\""));
@@ -223,23 +234,39 @@ public class RestTemplateIntegrationTests extends AbstractJettyServerTestCase {
 		MappingJacksonValue jacksonValue = new MappingJacksonValue(bean);
 		jacksonValue.setSerializationView(MyJacksonView1.class);
 		HttpEntity<MappingJacksonValue> entity = new HttpEntity<MappingJacksonValue>(jacksonValue, entityHeaders);
-		String s = template.postForObject(baseUrl + "/jsonpost", entity, String.class, "post");
+		String s = template.postForObject(baseUrl + "/jsonpost", entity, String.class);
 		assertTrue(s.contains("\"with1\":\"with\""));
 		assertFalse(s.contains("\"with2\":\"with\""));
 		assertFalse(s.contains("\"without\":\"without\""));
 	}
 
-	// SPR-12123
-
-	@Test
+	@Test  // SPR-12123
 	public void serverPort() {
 		String s = template.getForObject("http://localhost:{port}/get", String.class, port);
 		assertEquals("Invalid content", helloWorld, s);
 	}
 
+	@Test  // SPR-13154
+	public void jsonPostForObjectWithJacksonTypeInfoList() throws URISyntaxException {
+		List<ParentClass> list = new ArrayList<>();
+		list.add(new Foo("foo"));
+		list.add(new Bar("bar"));
+		ParameterizedTypeReference<?> typeReference = new ParameterizedTypeReference<List<ParentClass>>() {};
+		RequestEntity<List<ParentClass>> entity = RequestEntity
+				.post(new URI(baseUrl + "/jsonpost"))
+				.contentType(new MediaType("application", "json", Charset.forName("UTF-8")))
+				.body(list, typeReference.getType());
+		String content = template.exchange(entity, String.class).getBody();
+		assertTrue(content.contains("\"type\":\"foo\""));
+		assertTrue(content.contains("\"type\":\"bar\""));
+	}
+
+
 	public interface MyJacksonView1 {};
+
 	public interface MyJacksonView2 {};
 
+
 	public static class MySampleBean {
 
 		@JsonView(MyJacksonView1.class)
@@ -284,4 +311,50 @@ public class RestTemplateIntegrationTests extends AbstractJettyServerTestCase {
 		}
 	}
 
+
+	@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY, property = "type")
+	public static class ParentClass {
+
+		private String parentProperty;
+
+		public ParentClass() {
+		}
+
+		public ParentClass(String parentProperty) {
+			this.parentProperty = parentProperty;
+		}
+
+		public String getParentProperty() {
+			return parentProperty;
+		}
+
+		public void setParentProperty(String parentProperty) {
+			this.parentProperty = parentProperty;
+		}
+	}
+
+
+	@JsonTypeName("foo")
+	public static class Foo extends ParentClass {
+
+		public Foo() {
+		}
+
+		public Foo(String parentProperty) {
+			super(parentProperty);
+		}
+	}
+
+
+	@JsonTypeName("bar")
+	public static class Bar extends ParentClass {
+
+		public Bar() {
+		}
+
+		public Bar(String parentProperty) {
+			super(parentProperty);
+		}
+	}
+
 }
