@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,7 +26,8 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletRequestWrapper;
 import javax.servlet.http.HttpServletResponse;
 
-import junit.framework.TestCase;
+import org.junit.Before;
+import org.junit.Test;
 
 import org.springframework.beans.MutablePropertyValues;
 import org.springframework.beans.PropertyValue;
@@ -34,6 +35,7 @@ import org.springframework.context.ApplicationContextInitializer;
 import org.springframework.context.ConfigurableApplicationContext;
 import org.springframework.core.env.ConfigurableEnvironment;
 import org.springframework.core.env.DummyEnvironment;
+import org.springframework.http.HttpHeaders;
 import org.springframework.mock.web.test.MockHttpServletRequest;
 import org.springframework.mock.web.test.MockHttpServletResponse;
 import org.springframework.mock.web.test.MockServletConfig;
@@ -59,27 +61,29 @@ import org.springframework.web.util.WebUtils;
 
 import static org.hamcrest.CoreMatchers.*;
 import static org.junit.Assert.*;
-import static org.mockito.BDDMockito.*;
+import static org.mockito.Matchers.*;
+import static org.mockito.Mockito.*;
 
 /**
  * @author Rod Johnson
  * @author Juergen Hoeller
  * @author Rob Harrop
+ * @author Sam Brannen
  */
-public class DispatcherServletTests extends TestCase {
+public class DispatcherServletTests {
 
 	private static final String URL_KNOWN_ONLY_PARENT = "/knownOnlyToParent.do";
 
-	private MockServletConfig servletConfig;
+	private final MockServletConfig servletConfig = new MockServletConfig(new MockServletContext(), "simple");
 
 	private DispatcherServlet simpleDispatcherServlet;
 
 	private DispatcherServlet complexDispatcherServlet;
 
-	@Override
-	protected void setUp() throws ServletException {
-		servletConfig = new MockServletConfig(new MockServletContext(), "simple");
-		MockServletConfig complexConfig = new MockServletConfig(servletConfig.getServletContext(), "complex");
+
+	@Before
+	public void setUp() throws ServletException {
+		MockServletConfig complexConfig = new MockServletConfig(getServletContext(), "complex");
 		complexConfig.addInitParameter("publishContext", "false");
 		complexConfig.addInitParameter("class", "notWritable");
 		complexConfig.addInitParameter("unknownParam", "someValue");
@@ -99,14 +103,16 @@ public class DispatcherServletTests extends TestCase {
 		return servletConfig.getServletContext();
 	}
 
-	public void testDispatcherServletGetServletNameDoesNotFailWithoutConfig() {
+	@Test
+	public void dispatcherServletGetServletNameDoesNotFailWithoutConfig() {
 		DispatcherServlet ds = new DispatcherServlet();
 		assertNull(ds.getServletConfig());
 		assertNull(ds.getServletName());
 		assertNull(ds.getServletContext());
 	}
 
-	public void testConfiguredDispatcherServlets() {
+	@Test
+	public void configuredDispatcherServlets() {
 		assertTrue("Correct namespace",
 				("simple" + FrameworkServlet.DEFAULT_NAMESPACE_SUFFIX).equals(simpleDispatcherServlet.getNamespace()));
 		assertTrue("Correct attribute", (FrameworkServlet.SERVLET_CONTEXT_PREFIX + "simple").equals(
@@ -124,7 +130,8 @@ public class DispatcherServletTests extends TestCase {
 		complexDispatcherServlet.destroy();
 	}
 
-	public void testInvalidRequest() throws Exception {
+	@Test
+	public void invalidRequest() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/invalid.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 		simpleDispatcherServlet.service(request, response);
@@ -132,7 +139,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("correct error code", response.getStatus() == HttpServletResponse.SC_NOT_FOUND);
 	}
 
-	public void testRequestHandledEvent() throws Exception {
+	@Test
+	public void requestHandledEvent() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 		complexDispatcherServlet.service(request, response);
@@ -142,7 +150,8 @@ public class DispatcherServletTests extends TestCase {
 		assertEquals(1, listener.counter);
 	}
 
-	public void testPublishEventsOff() throws Exception {
+	@Test
+	public void publishEventsOff() throws Exception {
 		complexDispatcherServlet.setPublishEvents(false);
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
@@ -153,7 +162,8 @@ public class DispatcherServletTests extends TestCase {
 		assertEquals(0, listener.counter);
 	}
 
-	public void testParameterizableViewController() throws Exception {
+	@Test
+	public void parameterizableViewController() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/view.do");
 		request.addUserRole("role1");
 		MockHttpServletResponse response = new MockHttpServletResponse();
@@ -161,7 +171,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("forwarded to form", "myform.jsp".equals(response.getForwardedUrl()));
 	}
 
-	public void testHandlerInterceptorSuppressesView() throws Exception {
+	@Test
+	public void handlerInterceptorSuppressesView() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/view.do");
 		request.addUserRole("role1");
 		request.addParameter("noView", "true");
@@ -170,16 +181,18 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Not forwarded", response.getForwardedUrl() == null);
 	}
 
-	public void testLocaleRequest() throws Exception {
+	@Test
+	public void localeRequest() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.CANADA);
 		MockHttpServletResponse response = new MockHttpServletResponse();
 		simpleDispatcherServlet.service(request, response);
 		assertTrue("Not forwarded", response.getForwardedUrl() == null);
-		assertEquals("98", response.getHeader("Last-Modified"));
+		assertEquals("Wed, 01 Apr 2015 00:00:00 GMT", response.getHeader("Last-Modified"));
 	}
 
-	public void testUnknownRequest() throws Exception {
+	@Test
+	public void unknownRequest() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/unknown.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 		complexDispatcherServlet.service(request, response);
@@ -187,7 +200,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception").getClass().equals(ServletException.class));
 	}
 
-	public void testAnotherLocaleRequest() throws Exception {
+	@Test
+	public void anotherLocaleRequest() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do;abc=def");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -204,10 +218,11 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue(request.getAttribute("test3") != null);
 		assertTrue(request.getAttribute("test3x") != null);
 		assertTrue(request.getAttribute("test3y") != null);
-		assertEquals("99", response.getHeader("Last-Modified"));
+		assertEquals("Wed, 01 Apr 2015 00:00:01 GMT", response.getHeader("Last-Modified"));
 	}
 
-	public void testExistingMultipartRequest() throws Exception {
+	@Test
+	public void existingMultipartRequest() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do;abc=def");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -222,7 +237,8 @@ public class DispatcherServletTests extends TestCase {
 		assertNotNull(request.getAttribute("cleanedUp"));
 	}
 
-	public void testExistingMultipartRequestButWrapped() throws Exception {
+	@Test
+	public void existingMultipartRequestButWrapped() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do;abc=def");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -237,7 +253,8 @@ public class DispatcherServletTests extends TestCase {
 		assertNotNull(request.getAttribute("cleanedUp"));
 	}
 
-	public void testMultipartResolutionFailed() throws Exception {
+	@Test
+	public void multipartResolutionFailed() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do;abc=def");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -250,7 +267,8 @@ public class DispatcherServletTests extends TestCase {
 				SimpleMappingExceptionResolver.DEFAULT_EXCEPTION_ATTRIBUTE) instanceof MaxUploadSizeExceededException);
 	}
 
-	public void testHandlerInterceptorAbort() throws Exception {
+	@Test
+	public void handlerInterceptorAbort() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addParameter("abort", "true");
 		request.addPreferredLocale(Locale.CANADA);
@@ -266,7 +284,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue(request.getAttribute("test2y") == null);
 	}
 
-	public void testModelAndViewDefiningException() throws Exception {
+	@Test
+	public void modelAndViewDefiningException() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -282,7 +301,8 @@ public class DispatcherServletTests extends TestCase {
 		}
 	}
 
-	public void testSimpleMappingExceptionResolverWithSpecificHandler1() throws Exception {
+	@Test
+	public void simpleMappingExceptionResolverWithSpecificHandler1() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -294,7 +314,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception") instanceof IllegalAccessException);
 	}
 
-	public void testSimpleMappingExceptionResolverWithSpecificHandler2() throws Exception {
+	@Test
+	public void simpleMappingExceptionResolverWithSpecificHandler2() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -306,7 +327,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception") instanceof ServletException);
 	}
 
-	public void testSimpleMappingExceptionResolverWithAllHandlers1() throws Exception {
+	@Test
+	public void simpleMappingExceptionResolverWithAllHandlers1() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/loc.do");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -318,7 +340,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception") instanceof IllegalAccessException);
 	}
 
-	public void testSimpleMappingExceptionResolverWithAllHandlers2() throws Exception {
+	@Test
+	public void simpleMappingExceptionResolverWithAllHandlers2() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/loc.do");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -330,7 +353,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception") instanceof ServletException);
 	}
 
-	public void testSimpleMappingExceptionResolverWithDefaultErrorView() throws Exception {
+	@Test
+	public void simpleMappingExceptionResolverWithDefaultErrorView() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -342,7 +366,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception").getClass().equals(RuntimeException.class));
 	}
 
-	public void testLocaleChangeInterceptor1() throws Exception {
+	@Test
+	public void localeChangeInterceptor1() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.GERMAN);
 		request.addUserRole("role2");
@@ -354,7 +379,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception").getClass().equals(ServletException.class));
 	}
 
-	public void testLocaleChangeInterceptor2() throws Exception {
+	@Test
+	public void localeChangeInterceptor2() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.GERMAN);
 		request.addUserRole("role2");
@@ -365,7 +391,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Not forwarded", response.getForwardedUrl() == null);
 	}
 
-	public void testThemeChangeInterceptor1() throws Exception {
+	@Test
+	public void themeChangeInterceptor1() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -377,7 +404,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception").getClass().equals(ServletException.class));
 	}
 
-	public void testThemeChangeInterceptor2() throws Exception {
+	@Test
+	public void themeChangeInterceptor2() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.CANADA);
 		request.addUserRole("role1");
@@ -393,7 +421,8 @@ public class DispatcherServletTests extends TestCase {
 		}
 	}
 
-	public void testNotAuthorized() throws Exception {
+	@Test
+	public void notAuthorized() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/locale.do");
 		request.addPreferredLocale(Locale.CANADA);
 		MockHttpServletResponse response = new MockHttpServletResponse();
@@ -406,7 +435,8 @@ public class DispatcherServletTests extends TestCase {
 		}
 	}
 
-	public void testHeadMethodWithExplicitHandling() throws Exception {
+	@Test
+	public void headMethodWithExplicitHandling() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "HEAD", "/head.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 		complexDispatcherServlet.service(request, response);
@@ -418,7 +448,8 @@ public class DispatcherServletTests extends TestCase {
 		assertEquals("", response.getContentAsString());
 	}
 
-	public void testHeadMethodWithNoBodyResponse() throws Exception {
+	@Test
+	public void headMethodWithNoBodyResponse() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "HEAD", "/body.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 		complexDispatcherServlet.service(request, response);
@@ -430,7 +461,8 @@ public class DispatcherServletTests extends TestCase {
 		assertEquals("body", response.getContentAsString());
 	}
 
-	public void testNotDetectAllHandlerMappings() throws ServletException, IOException {
+	@Test
+	public void notDetectAllHandlerMappings() throws ServletException, IOException {
 		DispatcherServlet complexDispatcherServlet = new DispatcherServlet();
 		complexDispatcherServlet.setContextClass(ComplexWebApplicationContext.class);
 		complexDispatcherServlet.setNamespace("test");
@@ -443,7 +475,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue(response.getStatus() == HttpServletResponse.SC_NOT_FOUND);
 	}
 
-	public void testHandlerNotMappedWithAutodetect() throws ServletException, IOException {
+	@Test
+	public void handlerNotMappedWithAutodetect() throws ServletException, IOException {
 		DispatcherServlet complexDispatcherServlet = new DispatcherServlet();
 		// no parent
 		complexDispatcherServlet.setContextClass(ComplexWebApplicationContext.class);
@@ -456,10 +489,11 @@ public class DispatcherServletTests extends TestCase {
 		assertEquals(HttpServletResponse.SC_NOT_FOUND, response.getStatus());
 	}
 
-	public void testDetectHandlerMappingFromParent() throws ServletException, IOException {
+	@Test
+	public void detectHandlerMappingFromParent() throws ServletException, IOException {
 		// create a parent context that includes a mapping
 		StaticWebApplicationContext parent = new StaticWebApplicationContext();
-		parent.setServletContext(servletConfig.getServletContext());
+		parent.setServletContext(getServletContext());
 		parent.registerSingleton("parentHandler", ControllerFromParent.class, new MutablePropertyValues());
 
 		MutablePropertyValues pvs = new MutablePropertyValues();
@@ -485,7 +519,8 @@ public class DispatcherServletTests extends TestCase {
 				response.getStatus() == HttpServletResponse.SC_NOT_FOUND);
 	}
 
-	public void testDetectAllHandlerAdapters() throws ServletException, IOException {
+	@Test
+	public void detectAllHandlerAdapters() throws ServletException, IOException {
 		DispatcherServlet complexDispatcherServlet = new DispatcherServlet();
 		complexDispatcherServlet.setContextClass(ComplexWebApplicationContext.class);
 		complexDispatcherServlet.setNamespace("test");
@@ -501,7 +536,8 @@ public class DispatcherServletTests extends TestCase {
 		complexDispatcherServlet.service(request, response);
 	}
 
-	public void testNotDetectAllHandlerAdapters() throws ServletException, IOException {
+	@Test
+	public void notDetectAllHandlerAdapters() throws ServletException, IOException {
 		DispatcherServlet complexDispatcherServlet = new DispatcherServlet();
 		complexDispatcherServlet.setContextClass(ComplexWebApplicationContext.class);
 		complexDispatcherServlet.setNamespace("test");
@@ -522,7 +558,8 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("Exception exposed", request.getAttribute("exception").getClass().equals(ServletException.class));
 	}
 
-	public void testNotDetectAllHandlerExceptionResolvers() throws ServletException, IOException {
+	@Test
+	public void notDetectAllHandlerExceptionResolvers() throws ServletException, IOException {
 		DispatcherServlet complexDispatcherServlet = new DispatcherServlet();
 		complexDispatcherServlet.setContextClass(ComplexWebApplicationContext.class);
 		complexDispatcherServlet.setNamespace("test");
@@ -541,7 +578,8 @@ public class DispatcherServletTests extends TestCase {
 		}
 	}
 
-	public void testNotDetectAllViewResolvers() throws ServletException, IOException {
+	@Test
+	public void notDetectAllViewResolvers() throws ServletException, IOException {
 		DispatcherServlet complexDispatcherServlet = new DispatcherServlet();
 		complexDispatcherServlet.setContextClass(ComplexWebApplicationContext.class);
 		complexDispatcherServlet.setNamespace("test");
@@ -560,7 +598,8 @@ public class DispatcherServletTests extends TestCase {
 		}
 	}
 
-	public void testThrowExceptionIfNoHandlerFound() throws ServletException, IOException {
+	@Test
+	public void throwExceptionIfNoHandlerFound() throws ServletException, IOException {
 		DispatcherServlet complexDispatcherServlet = new DispatcherServlet();
 		complexDispatcherServlet.setContextClass(SimpleWebApplicationContext.class);
 		complexDispatcherServlet.setNamespace("test");
@@ -574,7 +613,19 @@ public class DispatcherServletTests extends TestCase {
 		assertTrue("correct error code", response.getStatus() == HttpServletResponse.SC_NOT_FOUND);
 	}
 
-	public void testCleanupAfterIncludeWithRemove() throws ServletException, IOException {
+	// SPR-12984
+
+	@Test
+	public void noHandlerFoundExceptionMessage() {
+		HttpHeaders headers = new HttpHeaders();
+		headers.add("foo", "bar");
+		NoHandlerFoundException ex = new NoHandlerFoundException("GET", "/foo", headers);
+		assertTrue(!ex.getMessage().contains("bar"));
+		assertTrue(!ex.toString().contains("bar"));
+	}
+
+	@Test
+	public void cleanupAfterIncludeWithRemove() throws ServletException, IOException {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/main.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 
@@ -593,7 +644,8 @@ public class DispatcherServletTests extends TestCase {
 		assertNull(request.getAttribute("command"));
 	}
 
-	public void testCleanupAfterIncludeWithRestore() throws ServletException, IOException {
+	@Test
+	public void cleanupAfterIncludeWithRestore() throws ServletException, IOException {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/main.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 
@@ -612,7 +664,8 @@ public class DispatcherServletTests extends TestCase {
 		assertSame(wac, request.getAttribute(DispatcherServlet.WEB_APPLICATION_CONTEXT_ATTRIBUTE));
 	}
 
-	public void testNoCleanupAfterInclude() throws ServletException, IOException {
+	@Test
+	public void noCleanupAfterInclude() throws ServletException, IOException {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/main.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 
@@ -632,7 +685,8 @@ public class DispatcherServletTests extends TestCase {
 		assertSame(wac, request.getAttribute(DispatcherServlet.WEB_APPLICATION_CONTEXT_ATTRIBUTE));
 	}
 
-	public void testServletHandlerAdapter() throws Exception {
+	@Test
+	public void servletHandlerAdapter() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "GET", "/servlet.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
 		complexDispatcherServlet.service(request, response);
@@ -645,7 +699,9 @@ public class DispatcherServletTests extends TestCase {
 		assertNull(myServlet.getServletConfig());
 	}
 
-	public void testWebApplicationContextLookup() {
+	@Test
+	@SuppressWarnings("deprecation")
+	public void webApplicationContextLookup() {
 		MockServletContext servletContext = new MockServletContext();
 		MockHttpServletRequest request = new MockHttpServletRequest(servletContext, "GET", "/invalid.do");
 
@@ -675,7 +731,8 @@ public class DispatcherServletTests extends TestCase {
 		}
 	}
 
-	public void testWithNoView() throws Exception {
+	@Test
+	public void withNoView() throws Exception {
 		MockServletContext servletContext = new MockServletContext();
 		MockHttpServletRequest request = new MockHttpServletRequest(servletContext, "GET", "/noview.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
@@ -684,7 +741,8 @@ public class DispatcherServletTests extends TestCase {
 		assertEquals("noview.jsp", response.getForwardedUrl());
 	}
 
-	public void testWithNoViewNested() throws Exception {
+	@Test
+	public void withNoViewNested() throws Exception {
 		MockServletContext servletContext = new MockServletContext();
 		MockHttpServletRequest request = new MockHttpServletRequest(servletContext, "GET", "/noview/simple.do");
 		MockHttpServletResponse response = new MockHttpServletResponse();
@@ -693,7 +751,8 @@ public class DispatcherServletTests extends TestCase {
 		assertEquals("noview/simple.jsp", response.getForwardedUrl());
 	}
 
-	public void testWithNoViewAndSamePath() throws Exception {
+	@Test
+	public void withNoViewAndSamePath() throws Exception {
 		InternalResourceViewResolver vr = (InternalResourceViewResolver) complexDispatcherServlet
 				.getWebApplicationContext().getBean("viewResolver2");
 		vr.setSuffix("");
@@ -711,7 +770,8 @@ public class DispatcherServletTests extends TestCase {
 		}
 	}
 
-	public void testDispatcherServletRefresh() throws ServletException {
+	@Test
+	public void dispatcherServletRefresh() throws ServletException {
 		MockServletContext servletContext = new MockServletContext("org/springframework/web/context");
 		DispatcherServlet servlet = new DispatcherServlet();
 
@@ -741,7 +801,8 @@ public class DispatcherServletTests extends TestCase {
 		servlet.destroy();
 	}
 
-	public void testDispatcherServletContextRefresh() throws ServletException {
+	@Test
+	public void dispatcherServletContextRefresh() throws ServletException {
 		MockServletContext servletContext = new MockServletContext("org/springframework/web/context");
 		DispatcherServlet servlet = new DispatcherServlet();
 
@@ -771,7 +832,8 @@ public class DispatcherServletTests extends TestCase {
 		servlet.destroy();
 	}
 
-	public void testEnvironmentOperations() {
+	@Test
+	public void environmentOperations() {
 		DispatcherServlet servlet = new DispatcherServlet();
 		ConfigurableEnvironment defaultEnv = servlet.getEnvironment();
 		assertThat(defaultEnv, notNullValue());
@@ -795,53 +857,59 @@ public class DispatcherServletTests extends TestCase {
 		assertThat(custom.getEnvironment(), instanceOf(CustomServletEnvironment.class));
 	}
 
-	public void testAllowedOptionsIncludesPatchMethod() throws Exception {
+	@Test
+	public void allowedOptionsIncludesPatchMethod() throws Exception {
 		MockHttpServletRequest request = new MockHttpServletRequest(getServletContext(), "OPTIONS", "/foo");
 		MockHttpServletResponse response = spy(new MockHttpServletResponse());
 		DispatcherServlet servlet = new DispatcherServlet();
+		servlet.setDispatchOptionsRequest(false);
 		servlet.service(request, response);
 		verify(response, never()).getHeader(anyString()); // SPR-10341
 		assertThat(response.getHeader("Allow"), equalTo("GET, HEAD, POST, PUT, DELETE, TRACE, OPTIONS, PATCH"));
 	}
 
-	public void testContextInitializers() throws Exception {
+	@Test
+	public void contextInitializers() throws Exception {
 		DispatcherServlet servlet = new DispatcherServlet();
 		servlet.setContextClass(SimpleWebApplicationContext.class);
 		servlet.setContextInitializers(new TestWebContextInitializer(), new OtherWebContextInitializer());
 		servlet.init(servletConfig);
-		assertEquals("true", servletConfig.getServletContext().getAttribute("initialized"));
-		assertEquals("true", servletConfig.getServletContext().getAttribute("otherInitialized"));
+		assertEquals("true", getServletContext().getAttribute("initialized"));
+		assertEquals("true", getServletContext().getAttribute("otherInitialized"));
 	}
 
-	public void testContextInitializerClasses() throws Exception {
+	@Test
+	public void contextInitializerClasses() throws Exception {
 		DispatcherServlet servlet = new DispatcherServlet();
 		servlet.setContextClass(SimpleWebApplicationContext.class);
 		servlet.setContextInitializerClasses(
 				TestWebContextInitializer.class.getName() + "," + OtherWebContextInitializer.class.getName());
 		servlet.init(servletConfig);
-		assertEquals("true", servletConfig.getServletContext().getAttribute("initialized"));
-		assertEquals("true", servletConfig.getServletContext().getAttribute("otherInitialized"));
+		assertEquals("true", getServletContext().getAttribute("initialized"));
+		assertEquals("true", getServletContext().getAttribute("otherInitialized"));
 	}
 
-	public void testGlobalInitializerClasses() throws Exception {
+	@Test
+	public void globalInitializerClasses() throws Exception {
 		DispatcherServlet servlet = new DispatcherServlet();
 		servlet.setContextClass(SimpleWebApplicationContext.class);
-		servletConfig.getServletContext().setInitParameter(ContextLoader.GLOBAL_INITIALIZER_CLASSES_PARAM,
+		getServletContext().setInitParameter(ContextLoader.GLOBAL_INITIALIZER_CLASSES_PARAM,
 				TestWebContextInitializer.class.getName() + "," + OtherWebContextInitializer.class.getName());
 		servlet.init(servletConfig);
-		assertEquals("true", servletConfig.getServletContext().getAttribute("initialized"));
-		assertEquals("true", servletConfig.getServletContext().getAttribute("otherInitialized"));
+		assertEquals("true", getServletContext().getAttribute("initialized"));
+		assertEquals("true", getServletContext().getAttribute("otherInitialized"));
 	}
 
-	public void testMixedInitializerClasses() throws Exception {
+	@Test
+	public void mixedInitializerClasses() throws Exception {
 		DispatcherServlet servlet = new DispatcherServlet();
 		servlet.setContextClass(SimpleWebApplicationContext.class);
-		servletConfig.getServletContext().setInitParameter(ContextLoader.GLOBAL_INITIALIZER_CLASSES_PARAM,
+		getServletContext().setInitParameter(ContextLoader.GLOBAL_INITIALIZER_CLASSES_PARAM,
 				TestWebContextInitializer.class.getName());
 		servlet.setContextInitializerClasses(OtherWebContextInitializer.class.getName());
 		servlet.init(servletConfig);
-		assertEquals("true", servletConfig.getServletContext().getAttribute("initialized"));
-		assertEquals("true", servletConfig.getServletContext().getAttribute("otherInitialized"));
+		assertEquals("true", getServletContext().getAttribute("initialized"));
+		assertEquals("true", getServletContext().getAttribute("otherInitialized"));
 	}
 
 
