@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2015 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,14 +29,18 @@ import java.util.Date;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
-
+import java.util.concurrent.TimeUnit;
+import java.util.stream.Collectors;
 import javax.servlet.RequestDispatcher;
 import javax.validation.constraints.NotNull;
 
+import com.fasterxml.jackson.databind.DeserializationFeature;
+import com.fasterxml.jackson.databind.MapperFeature;
+import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.dataformat.xml.XmlMapper;
+import org.apache.tiles.definition.UnresolvingLocaleDefinitionsFactory;
 import org.hamcrest.Matchers;
-
 import org.joda.time.LocalDate;
-
 import org.junit.Before;
 import org.junit.Test;
 
@@ -54,6 +58,7 @@ import org.springframework.format.annotation.DateTimeFormat;
 import org.springframework.format.annotation.DateTimeFormat.ISO;
 import org.springframework.format.annotation.NumberFormat;
 import org.springframework.format.support.FormattingConversionServiceFactoryBean;
+import org.springframework.http.CacheControl;
 import org.springframework.http.MediaType;
 import org.springframework.http.converter.HttpMessageConverter;
 import org.springframework.http.converter.json.AbstractJackson2HttpMessageConverter;
@@ -81,6 +86,7 @@ import org.springframework.web.context.request.async.CallableProcessingIntercept
 import org.springframework.web.context.request.async.DeferredResultProcessingInterceptor;
 import org.springframework.web.context.request.async.DeferredResultProcessingInterceptorAdapter;
 import org.springframework.web.context.support.GenericWebApplicationContext;
+import org.springframework.web.cors.CorsConfiguration;
 import org.springframework.web.method.HandlerMethod;
 import org.springframework.web.method.support.CompositeUriComponentsContributor;
 import org.springframework.web.method.support.InvocableHandlerMethod;
@@ -88,6 +94,7 @@ import org.springframework.web.servlet.HandlerExecutionChain;
 import org.springframework.web.servlet.HandlerInterceptor;
 import org.springframework.web.servlet.ModelAndView;
 import org.springframework.web.servlet.ViewResolver;
+import org.springframework.web.servlet.handler.AbstractHandlerMapping;
 import org.springframework.web.servlet.handler.BeanNameUrlHandlerMapping;
 import org.springframework.web.servlet.handler.ConversionServiceExposingInterceptor;
 import org.springframework.web.servlet.handler.MappedInterceptor;
@@ -116,6 +123,7 @@ import org.springframework.web.servlet.resource.ResourceTransformer;
 import org.springframework.web.servlet.resource.ResourceUrlProvider;
 import org.springframework.web.servlet.resource.ResourceUrlProviderExposingInterceptor;
 import org.springframework.web.servlet.resource.VersionResourceResolver;
+import org.springframework.web.servlet.resource.WebJarsResourceResolver;
 import org.springframework.web.servlet.theme.ThemeChangeInterceptor;
 import org.springframework.web.servlet.view.BeanNameViewResolver;
 import org.springframework.web.servlet.view.ContentNegotiatingViewResolver;
@@ -127,18 +135,17 @@ import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;
 import org.springframework.web.servlet.view.freemarker.FreeMarkerViewResolver;
 import org.springframework.web.servlet.view.groovy.GroovyMarkupConfigurer;
 import org.springframework.web.servlet.view.groovy.GroovyMarkupViewResolver;
+import org.springframework.web.servlet.view.script.ScriptTemplateConfigurer;
+import org.springframework.web.servlet.view.script.ScriptTemplateViewResolver;
+import org.springframework.web.servlet.view.tiles3.SpringBeanPreparerFactory;
 import org.springframework.web.servlet.view.tiles3.TilesConfigurer;
 import org.springframework.web.servlet.view.tiles3.TilesViewResolver;
 import org.springframework.web.servlet.view.velocity.VelocityConfigurer;
 import org.springframework.web.servlet.view.velocity.VelocityViewResolver;
 import org.springframework.web.util.UrlPathHelper;
 
-import com.fasterxml.jackson.databind.DeserializationFeature;
-import com.fasterxml.jackson.databind.MapperFeature;
-import com.fasterxml.jackson.databind.ObjectMapper;
-import com.fasterxml.jackson.dataformat.xml.XmlMapper;
-
-import static org.hamcrest.CoreMatchers.*;
+import static org.hamcrest.CoreMatchers.instanceOf;
+import static org.hamcrest.Matchers.*;
 import static org.junit.Assert.*;
 
 /**
@@ -181,7 +188,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testDefaultConfig() throws Exception {
-		loadBeanDefinitions("mvc-config.xml", 13);
+		loadBeanDefinitions("mvc-config.xml", 14);
 
 		RequestMappingHandlerMapping mapping = appContext.getBean(RequestMappingHandlerMapping.class);
 		assertNotNull(mapping);
@@ -202,7 +209,7 @@ public class MvcNamespaceTests {
 		assertTrue(converters.size() > 0);
 		for (HttpMessageConverter<?> converter : converters) {
 			if (converter instanceof AbstractJackson2HttpMessageConverter) {
-				ObjectMapper objectMapper = ((AbstractJackson2HttpMessageConverter)converter).getObjectMapper();
+				ObjectMapper objectMapper = ((AbstractJackson2HttpMessageConverter) converter).getObjectMapper();
 				assertFalse(objectMapper.getDeserializationConfig().isEnabled(MapperFeature.DEFAULT_VIEW_INCLUSION));
 				assertFalse(objectMapper.getSerializationConfig().isEnabled(MapperFeature.DEFAULT_VIEW_INCLUSION));
 				assertFalse(objectMapper.getDeserializationConfig().isEnabled(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES));
@@ -233,7 +240,7 @@ public class MvcNamespaceTests {
 		adapter.handle(request, response, handlerMethod);
 		assertTrue(handler.recordedValidationError);
 		assertEquals(LocalDate.parse("2009-10-31").toDate(), handler.date);
-		assertEquals(Double.valueOf(0.9999),handler.percent);
+		assertEquals(Double.valueOf(0.9999), handler.percent);
 
 		CompositeUriComponentsContributor uriComponentsContributor = this.appContext.getBean(
 				MvcUriComponentsBuilder.MVC_URI_COMPONENTS_CONTRIBUTOR_BEAN_NAME,
@@ -242,9 +249,9 @@ public class MvcNamespaceTests {
 		assertNotNull(uriComponentsContributor);
 	}
 
-	@Test(expected=TypeMismatchException.class)
+	@Test(expected = TypeMismatchException.class)
 	public void testCustomConversionService() throws Exception {
-		loadBeanDefinitions("mvc-config-custom-conversion-service.xml", 13);
+		loadBeanDefinitions("mvc-config-custom-conversion-service.xml", 14);
 
 		RequestMappingHandlerMapping mapping = appContext.getBean(RequestMappingHandlerMapping.class);
 		assertNotNull(mapping);
@@ -279,7 +286,7 @@ public class MvcNamespaceTests {
 	}
 
 	private void doTestCustomValidator(String xml) throws Exception {
-		loadBeanDefinitions(xml, 13);
+		loadBeanDefinitions(xml, 14);
 
 		RequestMappingHandlerMapping mapping = appContext.getBean(RequestMappingHandlerMapping.class);
 		assertNotNull(mapping);
@@ -301,7 +308,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testInterceptors() throws Exception {
-		loadBeanDefinitions("mvc-config-interceptors.xml", 20);
+		loadBeanDefinitions("mvc-config-interceptors.xml", 21);
 
 		RequestMappingHandlerMapping mapping = appContext.getBean(RequestMappingHandlerMapping.class);
 		assertNotNull(mapping);
@@ -337,17 +344,21 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testResources() throws Exception {
-		loadBeanDefinitions("mvc-config-resources.xml", 9);
+		loadBeanDefinitions("mvc-config-resources.xml", 20);
 
 		HttpRequestHandlerAdapter adapter = appContext.getBean(HttpRequestHandlerAdapter.class);
 		assertNotNull(adapter);
 
+		RequestMappingHandlerMapping mapping = appContext.getBean(RequestMappingHandlerMapping.class);
+		ContentNegotiationManager manager = mapping.getContentNegotiationManager();
+
 		ResourceHttpRequestHandler handler = appContext.getBean(ResourceHttpRequestHandler.class);
 		assertNotNull(handler);
+		assertSame(manager, handler.getContentNegotiationManager());
 
-		SimpleUrlHandlerMapping mapping = appContext.getBean(SimpleUrlHandlerMapping.class);
-		assertNotNull(mapping);
-		assertEquals(Ordered.LOWEST_PRECEDENCE - 1, mapping.getOrder());
+		SimpleUrlHandlerMapping resourceMapping = appContext.getBean(SimpleUrlHandlerMapping.class);
+		assertNotNull(resourceMapping);
+		assertEquals(Ordered.LOWEST_PRECEDENCE - 1, resourceMapping.getOrder());
 
 		BeanNameUrlHandlerMapping beanNameMapping = appContext.getBean(BeanNameUrlHandlerMapping.class);
 		assertNotNull(beanNameMapping);
@@ -356,15 +367,19 @@ public class MvcNamespaceTests {
 		ResourceUrlProvider urlProvider = appContext.getBean(ResourceUrlProvider.class);
 		assertNotNull(urlProvider);
 
-		MappedInterceptor mappedInterceptor = appContext.getBean(MappedInterceptor.class);
-		assertNotNull(urlProvider);
-		assertEquals(ResourceUrlProviderExposingInterceptor.class, mappedInterceptor.getInterceptor().getClass());
+		Map<String, MappedInterceptor> beans = appContext.getBeansOfType(MappedInterceptor.class);
+		List<Class<?>> interceptors = beans.values().stream()
+				.map(mappedInterceptor -> mappedInterceptor.getInterceptor().getClass())
+				.collect(Collectors.toList());
+		assertThat(interceptors, containsInAnyOrder(ConversionServiceExposingInterceptor.class,
+				ResourceUrlProviderExposingInterceptor.class));
 
 		MockHttpServletRequest request = new MockHttpServletRequest();
 		request.setRequestURI("/resources/foo.css");
 		request.setMethod("GET");
 
-		HandlerExecutionChain chain = mapping.getHandler(request);
+		HandlerExecutionChain chain = resourceMapping.getHandler(request);
+		assertNotNull(chain);
 		assertTrue(chain.getHandler() instanceof ResourceHttpRequestHandler);
 
 		MockHttpServletResponse response = new MockHttpServletResponse();
@@ -377,14 +392,14 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testResourcesWithOptionalAttributes() throws Exception {
-		loadBeanDefinitions("mvc-config-resources-optional-attrs.xml", 9);
+		loadBeanDefinitions("mvc-config-resources-optional-attrs.xml", 10);
 
 		SimpleUrlHandlerMapping mapping = appContext.getBean(SimpleUrlHandlerMapping.class);
 		assertNotNull(mapping);
 		assertEquals(5, mapping.getOrder());
 		assertNotNull(mapping.getUrlMap().get("/resources/**"));
 
-		ResourceHttpRequestHandler handler = appContext.getBean((String)mapping.getUrlMap().get("/resources/**"),
+		ResourceHttpRequestHandler handler = appContext.getBean((String) mapping.getUrlMap().get("/resources/**"),
 				ResourceHttpRequestHandler.class);
 		assertNotNull(handler);
 		assertEquals(3600, handler.getCacheSeconds());
@@ -392,20 +407,21 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testResourcesWithResolversTransformers() throws Exception {
-		loadBeanDefinitions("mvc-config-resources-chain.xml", 10);
+		loadBeanDefinitions("mvc-config-resources-chain.xml", 11);
 
 		SimpleUrlHandlerMapping mapping = appContext.getBean(SimpleUrlHandlerMapping.class);
 		assertNotNull(mapping);
 		assertNotNull(mapping.getUrlMap().get("/resources/**"));
-		ResourceHttpRequestHandler handler = appContext.getBean((String)mapping.getUrlMap().get("/resources/**"),
+		ResourceHttpRequestHandler handler = appContext.getBean((String) mapping.getUrlMap().get("/resources/**"),
 				ResourceHttpRequestHandler.class);
 		assertNotNull(handler);
 
 		List<ResourceResolver> resolvers = handler.getResourceResolvers();
-		assertThat(resolvers, Matchers.hasSize(3));
+		assertThat(resolvers, Matchers.hasSize(4));
 		assertThat(resolvers.get(0), Matchers.instanceOf(CachingResourceResolver.class));
 		assertThat(resolvers.get(1), Matchers.instanceOf(VersionResourceResolver.class));
-		assertThat(resolvers.get(2), Matchers.instanceOf(PathResourceResolver.class));
+		assertThat(resolvers.get(2), Matchers.instanceOf(WebJarsResourceResolver.class));
+		assertThat(resolvers.get(3), Matchers.instanceOf(PathResourceResolver.class));
 
 		CachingResourceResolver cachingResolver = (CachingResourceResolver) resolvers.get(0);
 		assertThat(cachingResolver.getCache(), Matchers.instanceOf(ConcurrentMapCache.class));
@@ -430,15 +446,19 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testResourcesWithResolversTransformersCustom() throws Exception {
-		loadBeanDefinitions("mvc-config-resources-chain-no-auto.xml", 11);
+		loadBeanDefinitions("mvc-config-resources-chain-no-auto.xml", 12);
 
 		SimpleUrlHandlerMapping mapping = appContext.getBean(SimpleUrlHandlerMapping.class);
 		assertNotNull(mapping);
 		assertNotNull(mapping.getUrlMap().get("/resources/**"));
-		ResourceHttpRequestHandler handler = appContext.getBean((String)mapping.getUrlMap().get("/resources/**"),
+		ResourceHttpRequestHandler handler = appContext.getBean((String) mapping.getUrlMap().get("/resources/**"),
 				ResourceHttpRequestHandler.class);
 		assertNotNull(handler);
 
+		assertThat(handler.getCacheControl().getHeaderValue(),
+				Matchers.equalTo(CacheControl.maxAge(1, TimeUnit.HOURS)
+						.sMaxAge(30, TimeUnit.MINUTES).cachePublic().getHeaderValue()));
+
 		List<ResourceResolver> resolvers = handler.getResourceResolvers();
 		assertThat(resolvers, Matchers.hasSize(3));
 		assertThat(resolvers.get(0), Matchers.instanceOf(VersionResourceResolver.class));
@@ -459,7 +479,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testDefaultServletHandler() throws Exception {
-		loadBeanDefinitions("mvc-config-default-servlet.xml", 5);
+		loadBeanDefinitions("mvc-config-default-servlet.xml", 6);
 
 		HttpRequestHandlerAdapter adapter = appContext.getBean(HttpRequestHandlerAdapter.class);
 		assertNotNull(adapter);
@@ -485,7 +505,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testDefaultServletHandlerWithOptionalAttributes() throws Exception {
-		loadBeanDefinitions("mvc-config-default-servlet-optional-attrs.xml", 5);
+		loadBeanDefinitions("mvc-config-default-servlet-optional-attrs.xml", 6);
 
 		HttpRequestHandlerAdapter adapter = appContext.getBean(HttpRequestHandlerAdapter.class);
 		assertNotNull(adapter);
@@ -511,7 +531,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testBeanDecoration() throws Exception {
-		loadBeanDefinitions("mvc-config-bean-decoration.xml", 15);
+		loadBeanDefinitions("mvc-config-bean-decoration.xml", 16);
 
 		RequestMappingHandlerMapping mapping = appContext.getBean(RequestMappingHandlerMapping.class);
 		assertNotNull(mapping);
@@ -532,7 +552,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testViewControllers() throws Exception {
-		loadBeanDefinitions("mvc-config-view-controllers.xml", 18);
+		loadBeanDefinitions("mvc-config-view-controllers.xml", 19);
 
 		RequestMappingHandlerMapping mapping = appContext.getBean(RequestMappingHandlerMapping.class);
 		assertNotNull(mapping);
@@ -613,7 +633,7 @@ public class MvcNamespaceTests {
 	/** WebSphere gives trailing servlet path slashes by default!! */
 	@Test
 	public void testViewControllersOnWebSphere() throws Exception {
-		loadBeanDefinitions("mvc-config-view-controllers.xml", 18);
+		loadBeanDefinitions("mvc-config-view-controllers.xml", 19);
 
 		SimpleUrlHandlerMapping mapping2 = appContext.getBean(SimpleUrlHandlerMapping.class);
 		SimpleControllerHandlerAdapter adapter = appContext.getBean(SimpleControllerHandlerAdapter.class);
@@ -657,7 +677,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testViewControllersDefaultConfig() {
-		loadBeanDefinitions("mvc-config-view-controllers-minimal.xml", 6);
+		loadBeanDefinitions("mvc-config-view-controllers-minimal.xml", 7);
 
 		SimpleUrlHandlerMapping hm = this.appContext.getBean(SimpleUrlHandlerMapping.class);
 		assertNotNull(hm);
@@ -680,7 +700,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testContentNegotiationManager() throws Exception {
-		loadBeanDefinitions("mvc-config-content-negotiation-manager.xml", 14);
+		loadBeanDefinitions("mvc-config-content-negotiation-manager.xml", 15);
 
 		RequestMappingHandlerMapping mapping = appContext.getBean(RequestMappingHandlerMapping.class);
 		ContentNegotiationManager manager = mapping.getContentNegotiationManager();
@@ -701,7 +721,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testAsyncSupportOptions() throws Exception {
-		loadBeanDefinitions("mvc-config-async-support.xml", 14);
+		loadBeanDefinitions("mvc-config-async-support.xml", 15);
 
 		RequestMappingHandlerAdapter adapter = appContext.getBean(RequestMappingHandlerAdapter.class);
 		assertNotNull(adapter);
@@ -718,14 +738,14 @@ public class MvcNamespaceTests {
 				(DeferredResultProcessingInterceptor[]) fieldAccessor.getPropertyValue("deferredResultInterceptors");
 		assertEquals(1, deferredResultInterceptors.length);
 	}
-	
+
 	@Test
 	public void testViewResolution() throws Exception {
-		loadBeanDefinitions("mvc-config-view-resolution.xml", 6);
+		loadBeanDefinitions("mvc-config-view-resolution.xml", 7);
 
 		ViewResolverComposite compositeResolver = this.appContext.getBean(ViewResolverComposite.class);
 		assertNotNull(compositeResolver);
-		assertEquals("Actual: " + compositeResolver.getViewResolvers(), 8, compositeResolver.getViewResolvers().size());
+		assertEquals("Actual: " + compositeResolver.getViewResolvers(), 9, compositeResolver.getViewResolvers().size());
 		assertEquals(Ordered.LOWEST_PRECEDENCE, compositeResolver.getOrder());
 
 		List<ViewResolver> resolvers = compositeResolver.getViewResolvers();
@@ -760,8 +780,15 @@ public class MvcNamespaceTests {
 		assertEquals(".tpl", accessor.getPropertyValue("suffix"));
 		assertEquals(1024, accessor.getPropertyValue("cacheLimit"));
 
-		assertEquals(InternalResourceViewResolver.class, resolvers.get(6).getClass());
+		resolver = resolvers.get(6);
+		assertThat(resolver, instanceOf(ScriptTemplateViewResolver.class));
+		accessor = new DirectFieldAccessor(resolver);
+		assertEquals("", accessor.getPropertyValue("prefix"));
+		assertEquals("", accessor.getPropertyValue("suffix"));
+		assertEquals(1024, accessor.getPropertyValue("cacheLimit"));
+
 		assertEquals(InternalResourceViewResolver.class, resolvers.get(7).getClass());
+		assertEquals(InternalResourceViewResolver.class, resolvers.get(8).getClass());
 
 		TilesConfigurer tilesConfigurer = appContext.getBean(TilesConfigurer.class);
 		assertNotNull(tilesConfigurer);
@@ -771,12 +798,14 @@ public class MvcNamespaceTests {
 		};
 		accessor = new DirectFieldAccessor(tilesConfigurer);
 		assertArrayEquals(definitions, (String[]) accessor.getPropertyValue("definitions"));
-		assertTrue((boolean)accessor.getPropertyValue("checkRefresh"));
+		assertTrue((boolean) accessor.getPropertyValue("checkRefresh"));
+		assertEquals(UnresolvingLocaleDefinitionsFactory.class, accessor.getPropertyValue("definitionsFactoryClass"));
+		assertEquals(SpringBeanPreparerFactory.class, accessor.getPropertyValue("preparerFactoryClass"));
 
 		FreeMarkerConfigurer freeMarkerConfigurer = appContext.getBean(FreeMarkerConfigurer.class);
 		assertNotNull(freeMarkerConfigurer);
 		accessor = new DirectFieldAccessor(freeMarkerConfigurer);
-		assertArrayEquals(new String[]{"/", "/test"}, (String[]) accessor.getPropertyValue("templateLoaderPaths"));
+		assertArrayEquals(new String[] {"/", "/test"}, (String[]) accessor.getPropertyValue("templateLoaderPaths"));
 
 		VelocityConfigurer velocityConfigurer = appContext.getBean(VelocityConfigurer.class);
 		assertNotNull(velocityConfigurer);
@@ -788,11 +817,22 @@ public class MvcNamespaceTests {
 		assertEquals("/test", groovyMarkupConfigurer.getResourceLoaderPath());
 		assertTrue(groovyMarkupConfigurer.isAutoIndent());
 		assertFalse(groovyMarkupConfigurer.isCacheTemplates());
+
+		ScriptTemplateConfigurer scriptTemplateConfigurer = appContext.getBean(ScriptTemplateConfigurer.class);
+		assertNotNull(scriptTemplateConfigurer);
+		assertEquals("render", scriptTemplateConfigurer.getRenderFunction());
+		assertEquals(MediaType.TEXT_PLAIN_VALUE, scriptTemplateConfigurer.getContentType());
+		assertEquals("ISO-8859-1", scriptTemplateConfigurer.getCharset().name());
+		assertEquals("classpath:", scriptTemplateConfigurer.getResourceLoaderPath());
+		assertFalse(scriptTemplateConfigurer.isSharedEngine());
+		String[] scripts = { "org/springframework/web/servlet/view/script/nashorn/render.js" };
+		accessor = new DirectFieldAccessor(scriptTemplateConfigurer);
+		assertArrayEquals(scripts, (String[]) accessor.getPropertyValue("scripts"));
 	}
 
 	@Test
 	public void testViewResolutionWithContentNegotiation() throws Exception {
-		loadBeanDefinitions("mvc-config-view-resolution-content-negotiation.xml", 6);
+		loadBeanDefinitions("mvc-config-view-resolution-content-negotiation.xml", 7);
 
 		ViewResolverComposite compositeResolver = this.appContext.getBean(ViewResolverComposite.class);
 		assertNotNull(compositeResolver);
@@ -802,7 +842,7 @@ public class MvcNamespaceTests {
 		List<ViewResolver> resolvers = compositeResolver.getViewResolvers();
 		assertEquals(ContentNegotiatingViewResolver.class, resolvers.get(0).getClass());
 		ContentNegotiatingViewResolver cnvr = (ContentNegotiatingViewResolver) resolvers.get(0);
-		assertEquals(6, cnvr.getViewResolvers().size());
+		assertEquals(7, cnvr.getViewResolvers().size());
 		assertEquals(1, cnvr.getDefaultViews().size());
 		assertTrue(cnvr.isUseNotAcceptableStatusCode());
 
@@ -811,6 +851,7 @@ public class MvcNamespaceTests {
 		ContentNegotiationManager manager = (ContentNegotiationManager) accessor.getPropertyValue(beanName);
 		assertNotNull(manager);
 		assertSame(manager, this.appContext.getBean(ContentNegotiationManager.class));
+		assertSame(manager, this.appContext.getBean("mvcContentNegotiationManager"));
 	}
 
 	@Test
@@ -825,7 +866,7 @@ public class MvcNamespaceTests {
 
 	@Test
 	public void testPathMatchingHandlerMappings() throws Exception {
-		loadBeanDefinitions("mvc-config-path-matching-mappings.xml", 22);
+		loadBeanDefinitions("mvc-config-path-matching-mappings.xml", 23);
 
 		RequestMappingHandlerMapping requestMapping = appContext.getBean(RequestMappingHandlerMapping.class);
 		assertNotNull(requestMapping);
@@ -844,6 +885,59 @@ public class MvcNamespaceTests {
 		}
 	}
 
+	@Test
+	public void testCorsMinimal() throws Exception {
+		loadBeanDefinitions("mvc-config-cors-minimal.xml", 14);
+
+		String[] beanNames = appContext.getBeanNamesForType(AbstractHandlerMapping.class);
+		assertEquals(2, beanNames.length);
+		for (String beanName : beanNames) {
+			AbstractHandlerMapping handlerMapping = (AbstractHandlerMapping)appContext.getBean(beanName);
+			assertNotNull(handlerMapping);
+			Map<String, CorsConfiguration> configs = handlerMapping.getCorsConfigurations();
+			assertNotNull(configs);
+			assertEquals(1, configs.size());
+			CorsConfiguration config = configs.get("/**");
+			assertNotNull(config);
+			assertArrayEquals(new String[]{"*"}, config.getAllowedOrigins().toArray());
+			assertArrayEquals(new String[]{"GET", "HEAD", "POST"}, config.getAllowedMethods().toArray());
+			assertArrayEquals(new String[]{"*"}, config.getAllowedHeaders().toArray());
+			assertNull(config.getExposedHeaders());
+			assertTrue(config.getAllowCredentials());
+			assertEquals(new Long(1600), config.getMaxAge());
+		}
+	}
+
+	@Test
+	public void testCors() throws Exception {
+		loadBeanDefinitions("mvc-config-cors.xml", 14);
+
+		String[] beanNames = appContext.getBeanNamesForType(AbstractHandlerMapping.class);
+		assertEquals(2, beanNames.length);
+		for (String beanName : beanNames) {
+			AbstractHandlerMapping handlerMapping = (AbstractHandlerMapping)appContext.getBean(beanName);
+			assertNotNull(handlerMapping);
+			Map<String, CorsConfiguration> configs = handlerMapping.getCorsConfigurations();
+			assertNotNull(configs);
+			assertEquals(2, configs.size());
+			CorsConfiguration config = configs.get("/api/**");
+			assertNotNull(config);
+			assertArrayEquals(new String[]{"http://domain1.com", "http://domain2.com"}, config.getAllowedOrigins().toArray());
+			assertArrayEquals(new String[]{"GET", "PUT"}, config.getAllowedMethods().toArray());
+			assertArrayEquals(new String[]{"header1", "header2", "header3"}, config.getAllowedHeaders().toArray());
+			assertArrayEquals(new String[]{"header1", "header2"}, config.getExposedHeaders().toArray());
+			assertFalse(config.getAllowCredentials());
+			assertEquals(new Long(123), config.getMaxAge());
+			config = configs.get("/resources/**");
+			assertArrayEquals(new String[]{"http://domain1.com"}, config.getAllowedOrigins().toArray());
+			assertArrayEquals(new String[]{"GET", "HEAD", "POST"}, config.getAllowedMethods().toArray());
+			assertArrayEquals(new String[]{"*"}, config.getAllowedHeaders().toArray());
+			assertNull(config.getExposedHeaders());
+			assertTrue(config.getAllowCredentials());
+			assertEquals(new Long(1600), config.getMaxAge());
+		}
+	}
+
 
 	private void loadBeanDefinitions(String fileName, int expectedBeanCount) {
 		XmlBeanDefinitionReader reader = new XmlBeanDefinitionReader(appContext);
@@ -855,39 +949,47 @@ public class MvcNamespaceTests {
 	}
 
 
-	@DateTimeFormat(iso=ISO.DATE)
+	@DateTimeFormat(iso = ISO.DATE)
 	@Target({ElementType.PARAMETER})
 	@Retention(RetentionPolicy.RUNTIME)
 	public @interface IsoDate {
 	}
 
+
 	@NumberFormat(style = NumberFormat.Style.PERCENT)
 	@Target({ElementType.PARAMETER})
 	@Retention(RetentionPolicy.RUNTIME)
 	public @interface PercentNumber {
 	}
 
+
 	@Validated(MyGroup.class)
 	@Target({ElementType.PARAMETER})
 	@Retention(RetentionPolicy.RUNTIME)
 	public @interface MyValid {
 	}
 
+
 	@Controller
 	public static class TestController {
 
 		private Date date;
+
 		private Double percent;
+
 		private boolean recordedValidationError;
 
 		@RequestMapping
-		public void testBind(@RequestParam @IsoDate Date date, @RequestParam(required = false) @PercentNumber Double percent, @MyValid TestBean bean, BindingResult result) {
+		public void testBind(@RequestParam @IsoDate Date date,
+				@RequestParam(required = false) @PercentNumber Double percent,
+				@MyValid TestBean bean, BindingResult result) {
 			this.date = date;
 			this.percent = percent;
 			this.recordedValidationError = (result.getErrorCount() == 1);
 		}
 	}
 
+
 	public static class TestValidator implements Validator {
 
 		boolean validatorInvoked;
@@ -903,13 +1005,15 @@ public class MvcNamespaceTests {
 		}
 	}
 
+
 	@Retention(RetentionPolicy.RUNTIME)
 	public @interface MyGroup {
 	}
 
+
 	private static class TestBean {
 
-		@NotNull(groups=MyGroup.class)
+		@NotNull(groups = MyGroup.class)
 		private String field;
 
 		@SuppressWarnings("unused")
@@ -923,24 +1027,29 @@ public class MvcNamespaceTests {
 		}
 	}
 
+
 	private static class TestMockServletContext extends MockServletContext {
 
 		@Override
 		public RequestDispatcher getNamedDispatcher(String path) {
 			if (path.equals("default") || path.equals("custom")) {
 				return new MockRequestDispatcher("/");
-			} else {
+			}
+			else {
 				return null;
 			}
 		}
 	}
 
+
 	public static class TestCallableProcessingInterceptor extends CallableProcessingInterceptorAdapter {
 	}
 
+
 	public static class TestDeferredResultProcessingInterceptor extends DeferredResultProcessingInterceptorAdapter {
 	}
 
+
 	public static class TestPathMatcher implements PathMatcher {
 
 		@Override
@@ -979,9 +1088,11 @@ public class MvcNamespaceTests {
 		}
 	}
 
+
 	public static class TestPathHelper extends UrlPathHelper {
 	}
 
+
 	public static class TestCacheManager implements CacheManager {
 
 		@Override
