@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2014 the original author or authors.
+ * Copyright 2002-2016 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,10 @@
 
 package org.springframework.web.servlet.mvc.method.annotation;
 
+import java.lang.annotation.ElementType;
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
+import java.lang.annotation.Target;
 import java.lang.reflect.Method;
 import java.util.Arrays;
 import java.util.Collections;
@@ -23,15 +27,20 @@ import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
 
-import org.junit.Before;
 import org.junit.Test;
 
+import org.springframework.core.annotation.AliasFor;
 import org.springframework.http.MediaType;
 import org.springframework.stereotype.Controller;
-import org.springframework.util.StringValueResolver;
 import org.springframework.web.accept.ContentNegotiationManager;
 import org.springframework.web.accept.PathExtensionContentNegotiationStrategy;
+import org.springframework.web.bind.annotation.DeleteMapping;
+import org.springframework.web.bind.annotation.GetMapping;
+import org.springframework.web.bind.annotation.PatchMapping;
+import org.springframework.web.bind.annotation.PostMapping;
+import org.springframework.web.bind.annotation.PutMapping;
 import org.springframework.web.bind.annotation.RequestMapping;
+import org.springframework.web.bind.annotation.RequestMethod;
 import org.springframework.web.context.support.StaticWebApplicationContext;
 import org.springframework.web.servlet.mvc.method.RequestMappingInfo;
 
@@ -41,16 +50,15 @@ import static org.junit.Assert.*;
  * Tests for {@link RequestMappingHandlerMapping}.
  *
  * @author Rossen Stoyanchev
+ * @author Sam Brannen
  */
 public class RequestMappingHandlerMappingTests {
 
-	private RequestMappingHandlerMapping handlerMapping;
+	private final StaticWebApplicationContext wac = new StaticWebApplicationContext();
 
-
-	@Before
-	public void setup() {
-		this.handlerMapping = new RequestMappingHandlerMapping();
-		this.handlerMapping.setApplicationContext(new StaticWebApplicationContext());
+	private final RequestMappingHandlerMapping handlerMapping = new RequestMappingHandlerMapping();
+	{
+		this.handlerMapping.setApplicationContext(wac);
 	}
 
 
@@ -88,8 +96,7 @@ public class RequestMappingHandlerMappingTests {
 			}
 		};
 
-		StaticWebApplicationContext wac = new StaticWebApplicationContext();
-		wac.registerSingleton("testController", TestController.class);
+		wac.registerSingleton("testController", ComposedAnnotationController.class);
 		wac.refresh();
 
 		hm.setContentNegotiationManager(manager);
@@ -118,12 +125,9 @@ public class RequestMappingHandlerMappingTests {
 
 	@Test
 	public void resolveEmbeddedValuesInPatterns() {
-		this.handlerMapping.setEmbeddedValueResolver(new StringValueResolver() {
-			@Override
-			public String resolveStringValue(String value) {
-				return "/${pattern}/bar".equals(value) ? "/foo/bar" : value;
-			}
-		});
+		this.handlerMapping.setEmbeddedValueResolver(
+				value -> "/${pattern}/bar".equals(value) ? "/foo/bar" : value
+		);
 
 		String[] patterns = new String[] { "/foo", "/${pattern}/bar" };
 		String[] result = this.handlerMapping.resolveEmbeddedValuesInPatterns(patterns);
@@ -131,13 +135,111 @@ public class RequestMappingHandlerMappingTests {
 		assertArrayEquals(new String[] { "/foo", "/foo/bar" }, result);
 	}
 
+	@Test
+	public void resolveRequestMappingViaComposedAnnotation() throws Exception {
+		RequestMappingInfo info = assertComposedAnnotationMapping("postJson", "/postJson", RequestMethod.POST);
+
+		assertEquals(MediaType.APPLICATION_JSON_VALUE,
+			info.getConsumesCondition().getConsumableMediaTypes().iterator().next().toString());
+		assertEquals(MediaType.APPLICATION_JSON_VALUE,
+			info.getProducesCondition().getProducibleMediaTypes().iterator().next().toString());
+	}
+
+	@Test
+	public void getMapping() throws Exception {
+		assertComposedAnnotationMapping(RequestMethod.GET);
+	}
+
+	@Test
+	public void postMapping() throws Exception {
+		assertComposedAnnotationMapping(RequestMethod.POST);
+	}
+
+	@Test
+	public void putMapping() throws Exception {
+		assertComposedAnnotationMapping(RequestMethod.PUT);
+	}
+
+	@Test
+	public void deleteMapping() throws Exception {
+		assertComposedAnnotationMapping(RequestMethod.DELETE);
+	}
+
+	@Test
+	public void patchMapping() throws Exception {
+		assertComposedAnnotationMapping(RequestMethod.PATCH);
+	}
+
+	private RequestMappingInfo assertComposedAnnotationMapping(RequestMethod requestMethod) throws Exception {
+		String methodName = requestMethod.name().toLowerCase();
+		String path = "/" + methodName;
+
+		return assertComposedAnnotationMapping(methodName, path, requestMethod);
+	}
+
+	private RequestMappingInfo assertComposedAnnotationMapping(String methodName, String path,
+			RequestMethod requestMethod) throws Exception {
+
+		Class<?> clazz = ComposedAnnotationController.class;
+		Method method = clazz.getMethod(methodName);
+		RequestMappingInfo info = this.handlerMapping.getMappingForMethod(method, clazz);
+
+		assertNotNull(info);
+
+		Set<String> paths = info.getPatternsCondition().getPatterns();
+		assertEquals(1, paths.size());
+		assertEquals(path, paths.iterator().next());
+
+		Set<RequestMethod> methods = info.getMethodsCondition().getMethods();
+		assertEquals(1, methods.size());
+		assertEquals(requestMethod, methods.iterator().next());
+
+		return info;
+	}
+
 
 	@Controller
-	static class TestController {
+	static class ComposedAnnotationController {
 
 		@RequestMapping
 		public void handle() {
 		}
+
+		@PostJson("/postJson")
+		public void postJson() {
+		}
+
+		@GetMapping("/get")
+		public void get() {
+		}
+
+		@PostMapping("/post")
+		public void post() {
+		}
+
+		@PutMapping("/put")
+		public void put() {
+		}
+
+		@DeleteMapping("/delete")
+		public void delete() {
+		}
+
+		@PatchMapping("/patch")
+		public void patch() {
+		}
+
+	}
+
+	@RequestMapping(method = RequestMethod.POST,
+			produces = MediaType.APPLICATION_JSON_VALUE,
+			consumes = MediaType.APPLICATION_JSON_VALUE)
+	@Target(ElementType.METHOD)
+	@Retention(RetentionPolicy.RUNTIME)
+	@interface PostJson {
+
+		@AliasFor(annotation = RequestMapping.class, attribute = "path")
+		String[] value() default {};
 	}
 
 }
